/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/gpl-3.0.txt
 *
 */
package com.frojasg1.chesspdfbrowser.application;

import com.frojasg1.chesspdfbrowser.application.JDial_applicationConfiguration.ComboBoxOfChessLanguageData;
import com.frojasg1.chesspdfbrowser.configuration.ApplicationConfiguration;
import com.frojasg1.chesspdfbrowser.configuration.AppStringsConf;
import com.frojasg1.chesspdfbrowser.configuration.ChessStringsConf;
import com.frojasg1.chesspdfbrowser.model.chess.configuration.ChessLanguageConfiguration;
import com.frojasg1.general.desktop.view.panels.AcceptCancelRevertControllerInterface;
import com.frojasg1.general.desktop.view.panels.AcceptCancelRevertPanel;
import com.hotmail.frojasg1.applications.common.components.internationalization.window.InternationalizedJDialog;
import com.frojasg1.applications.common.configuration.ConfigurationParent;
import com.frojasg1.general.ArrayFunctions;
import com.frojasg1.general.ResourceFunctions;
import com.frojasg1.general.desktop.view.panels.InformerInterface;
import com.frojasg1.general.dialogs.DialogsWrapper;
import com.frojasg1.general.exceptions.ValidateException;
import com.frojasg1.generic.GenericFunctions;
import java.awt.Component;
import java.io.File;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JTextField;

/**
 *
 * @author Usuario
 */
public class JDial_applicationConfiguration extends InternationalizedJDialog
										implements AcceptCancelRevertControllerInterface
{
	protected AcceptCancelRevertPanel _acceptPanel = null;

	protected boolean a_initializing = true;
	protected boolean a_userHasPressedOK = false;
	protected String a_languageInConstructor = null;
	protected final static String a_configurationBaseFileName = "JDial_applicationConfiguration";

	protected Map<Component, ComboBoxOfChessLanguageData> _mapComboBoxes = null;

	protected boolean _modifiedByProgram = false;

	protected String[] _arrayOfLanguages = null;
	
	/**
	 * Creates new form JDial_applicationConfiguration
	 */
	public JDial_applicationConfiguration(java.awt.Frame parent, boolean modal )
	{
		super(parent, modal, ApplicationConfiguration.instance() );
		initComponents();
		initOwnComponents();

		_appliConf = ApplicationConfiguration.instance();

		a_languageInConstructor = _appliConf.getLanguage();

		setWindowConfiguration();

		M_initializeComponentContents();

		boolean initialConfiguration = true;
		fillInComboBoxesOfChessLanguage( initialConfiguration );
//		float factor = ApplicationConfiguration.instance().M_getApplicationFontSize();

		if( modal )
			setAlwaysOnTop( true );
	}

	protected void initOwnComponents()
	{
		System.out.println( "jDial_applicationConfiguration" );
		_acceptPanel = new AcceptCancelRevertPanel( this );

		jPanel5.add( _acceptPanel );
		_acceptPanel.setBounds( 0, 0, jPanel5.getWidth(), jPanel5.getHeight() );
	}

	protected void setWindowConfiguration( )
	{
/*
		a_intern = new JFrameInternationalization(	ApplicationConfiguration.sa_MAIN_FOLDER,
													ApplicationConfiguration.sa_APPLICATION_NAME,
													ApplicationConfiguration.sa_CONFIGURATION_GROUP,
													ApplicationConfiguration.sa_PATH_PROPERTIES_IN_JAR,
													a_configurationBaseFileName,
													this,
													parent,
													a_vectorJpopupMenus,
													false,
													mapRRCI );
*/
		createInternationalization(	getAppliConf().getConfigurationMainFolder(),
									getAppliConf().getApplicationNameFolder(),
									getAppliConf().getApplicationGroup(),
									getAppliConf().getInternationalPropertiesPathInJar(),
									a_configurationBaseFileName,
									this,
									getParent(),
									null,
									true,
									null );

//		setIcon( "com/frojasg1/chesspdfbrowser/resources/icons/App.icon.png" );
		try
		{
			a_intern.changeLanguage( getAppliConf().getLanguage() );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jTP_tabbedPane = new javax.swing.JTabbedPane();
        jP_chessView = new javax.swing.JPanel();
        jCB_showComments = new javax.swing.JCheckBox();
        jCB_showNAGs = new javax.swing.JCheckBox();
        jL_languageToShowGames1 = new javax.swing.JLabel();
        jCB_languageToShowGames1 = new javax.swing.JComboBox();
        jTF_languageToShowGamesPiecesString1 = new javax.swing.JTextField();
        jCB_detachedWindowsAlwaysOnTop = new javax.swing.JCheckBox();
        jP_language = new javax.swing.JPanel();
        jP_applicationLanguage = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTP_additionalLanguageFoder = new javax.swing.JTextPane();
        jLbl_additionalLanguageFolder = new javax.swing.JLabel();
        jTf_additionalLanguage = new javax.swing.JTextField();
        jLbl_additionalLanguage1 = new javax.swing.JLabel();
        jCb_language = new javax.swing.JComboBox();
        jLbl_language = new javax.swing.JLabel();
        jB_addLanguage = new javax.swing.JButton();
        jP_languageForChessGames = new javax.swing.JPanel();
        jL_languageToParseGamesFrom = new javax.swing.JLabel();
        jCB_languageToParseGamesFrom = new javax.swing.JComboBox();
        jTF_languageToParseGamesFromPiecesString = new javax.swing.JTextField();
        jL_languageToShowGames = new javax.swing.JLabel();
        jCB_languageToShowGames = new javax.swing.JComboBox();
        jTF_languageToShowGamesPiecesString = new javax.swing.JTextField();
        jPanel5 = new javax.swing.JPanel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("Configuration window");
        setMinimumSize(new java.awt.Dimension(590, 455));
        setName("ConfigurationDialog"); // NOI18N
        setPreferredSize(new java.awt.Dimension(590, 455));
        setResizable(false);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });
        getContentPane().setLayout(null);

        jPanel1.setLayout(null);

        jTP_tabbedPane.setName("jTP_tabbedPane"); // NOI18N

        jP_chessView.setName("jP_chessView"); // NOI18N
        jP_chessView.setLayout(null);

        jCB_showComments.setText("Show comments");
        jCB_showComments.setName("jCB_showComments"); // NOI18N
        jP_chessView.add(jCB_showComments);
        jCB_showComments.setBounds(50, 40, 320, 23);

        jCB_showNAGs.setText("Show NAGs");
        jCB_showNAGs.setName("jCB_showNAGs"); // NOI18N
        jP_chessView.add(jCB_showNAGs);
        jCB_showNAGs.setBounds(50, 100, 340, 23);

        jL_languageToShowGames1.setFont(new java.awt.Font("Tahoma", 1, 12)); // NOI18N
        jL_languageToShowGames1.setText("Language to show games :");
        jL_languageToShowGames1.setName("jL_languageToShowGames"); // NOI18N
        jP_chessView.add(jL_languageToShowGames1);
        jL_languageToShowGames1.setBounds(50, 230, 270, 15);

        jCB_languageToShowGames1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCB_languageToShowGames1ActionPerformed(evt);
            }
        });
        jP_chessView.add(jCB_languageToShowGames1);
        jCB_languageToShowGames1.setBounds(50, 260, 170, 20);

        jTF_languageToShowGamesPiecesString1.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jTF_languageToShowGamesPiecesString1FocusLost(evt);
            }
        });
        jP_chessView.add(jTF_languageToShowGamesPiecesString1);
        jTF_languageToShowGamesPiecesString1.setBounds(230, 260, 80, 20);

        jCB_detachedWindowsAlwaysOnTop.setText("Show chess game detached windows always on top");
        jCB_detachedWindowsAlwaysOnTop.setName("jCB_detachedWindowsAlwaysOnTop"); // NOI18N
        jP_chessView.add(jCB_detachedWindowsAlwaysOnTop);
        jCB_detachedWindowsAlwaysOnTop.setBounds(50, 160, 500, 23);

        jTP_tabbedPane.addTab("Chess view", jP_chessView);

        jP_language.setName("jP_language"); // NOI18N
        jP_language.setLayout(null);

        jP_applicationLanguage.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)), "Application Language"));
        jP_applicationLanguage.setName("jP_applicationLanguage"); // NOI18N
        jP_applicationLanguage.setLayout(null);

        jTP_additionalLanguageFoder.setEditable(false);
        jScrollPane1.setViewportView(jTP_additionalLanguageFoder);

        jP_applicationLanguage.add(jScrollPane1);
        jScrollPane1.setBounds(30, 130, 520, 50);

        jLbl_additionalLanguageFolder.setFont(new java.awt.Font("Tahoma", 1, 12)); // NOI18N
        jLbl_additionalLanguageFolder.setText("Additional Language Folder:");
        jLbl_additionalLanguageFolder.setToolTipText("");
        jLbl_additionalLanguageFolder.setName("jLbl_additionalLanguageFolder"); // NOI18N
        jP_applicationLanguage.add(jLbl_additionalLanguageFolder);
        jLbl_additionalLanguageFolder.setBounds(110, 110, 430, 20);

        jTf_additionalLanguage.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jTf_additionalLanguageFocusLost(evt);
            }
        });
        jTf_additionalLanguage.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                jTf_additionalLanguageKeyReleased(evt);
            }
        });
        jP_applicationLanguage.add(jTf_additionalLanguage);
        jTf_additionalLanguage.setBounds(40, 70, 40, 20);

        jLbl_additionalLanguage1.setFont(new java.awt.Font("Tahoma", 1, 12)); // NOI18N
        jLbl_additionalLanguage1.setText("Additional Language");
        jLbl_additionalLanguage1.setName("jLbl_additionalLanguage"); // NOI18N
        jP_applicationLanguage.add(jLbl_additionalLanguage1);
        jLbl_additionalLanguage1.setBounds(100, 70, 230, 20);

        jCb_language.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        jCb_language.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCb_languageActionPerformed(evt);
            }
        });
        jP_applicationLanguage.add(jCb_language);
        jCb_language.setBounds(20, 30, 56, 20);

        jLbl_language.setFont(new java.awt.Font("Tahoma", 1, 12)); // NOI18N
        jLbl_language.setText("Language");
        jLbl_language.setName("jLbl_language"); // NOI18N
        jP_applicationLanguage.add(jLbl_language);
        jLbl_language.setBounds(110, 30, 430, 20);

        jB_addLanguage.setText("Add language");
        jB_addLanguage.setEnabled(false);
        jB_addLanguage.setName("jB_addLanguage"); // NOI18N
        jB_addLanguage.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jB_addLanguageActionPerformed(evt);
            }
        });
        jP_applicationLanguage.add(jB_addLanguage);
        jB_addLanguage.setBounds(330, 70, 120, 23);

        jP_language.add(jP_applicationLanguage);
        jP_applicationLanguage.setBounds(10, 10, 570, 190);

        jP_languageForChessGames.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)), "Language for chess game moves"));
        jP_languageForChessGames.setName("jP_languageForChessGames"); // NOI18N
        jP_languageForChessGames.setLayout(null);

        jL_languageToParseGamesFrom.setFont(new java.awt.Font("Tahoma", 1, 12)); // NOI18N
        jL_languageToParseGamesFrom.setText("Language to parse games from:");
        jL_languageToParseGamesFrom.setName("jL_languageToParseGamesFrom"); // NOI18N
        jP_languageForChessGames.add(jL_languageToParseGamesFrom);
        jL_languageToParseGamesFrom.setBounds(20, 30, 270, 15);

        jCB_languageToParseGamesFrom.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCB_languageToShowGames1ActionPerformed(evt);
            }
        });
        jP_languageForChessGames.add(jCB_languageToParseGamesFrom);
        jCB_languageToParseGamesFrom.setBounds(290, 30, 170, 20);
        jP_languageForChessGames.add(jTF_languageToParseGamesFromPiecesString);
        jTF_languageToParseGamesFromPiecesString.setBounds(470, 30, 80, 20);

        jL_languageToShowGames.setFont(new java.awt.Font("Tahoma", 1, 12)); // NOI18N
        jL_languageToShowGames.setText("Language to show games :");
        jL_languageToShowGames.setName("jL_languageToShowGames"); // NOI18N
        jP_languageForChessGames.add(jL_languageToShowGames);
        jL_languageToShowGames.setBounds(20, 60, 270, 15);

        jCB_languageToShowGames.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCB_languageToShowGames1ActionPerformed(evt);
            }
        });
        jP_languageForChessGames.add(jCB_languageToShowGames);
        jCB_languageToShowGames.setBounds(290, 60, 170, 20);

        jTF_languageToShowGamesPiecesString.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jTF_languageToShowGamesPiecesString1FocusLost(evt);
            }
        });
        jP_languageForChessGames.add(jTF_languageToShowGamesPiecesString);
        jTF_languageToShowGamesPiecesString.setBounds(470, 60, 80, 20);

        jP_language.add(jP_languageForChessGames);
        jP_languageForChessGames.setBounds(10, 210, 570, 100);

        jTP_tabbedPane.addTab("Language", jP_language);

        jPanel1.add(jTP_tabbedPane);
        jTP_tabbedPane.setBounds(0, 0, 590, 350);

        jPanel5.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 102, 204)));
        jPanel5.setPreferredSize(new java.awt.Dimension(130, 50));
        jPanel5.setLayout(null);
        jPanel1.add(jPanel5);
        jPanel5.setBounds(230, 360, 130, 50);

        getContentPane().add(jPanel1);
        jPanel1.setBounds(0, 0, 590, 420);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        // TODO add your handling code here:

		cancel( null );

    }//GEN-LAST:event_formWindowClosing

    private void jTF_languageToShowGamesPiecesString1FocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jTF_languageToShowGamesPiecesString1FocusLost
        // TODO add your handling code here:

        Object obj = evt.getSource();

        if( obj instanceof JTextField )
        {
            JTextField jtf = (JTextField) obj;

            ComboBoxOfChessLanguageData data = _mapComboBoxes.get( jtf );

            if( data != null )
            {
                data = _mapComboBoxes.get( data.getLinked() );

                if( data != null )
                {
                    JTextField jtf2 = data.getLinkedTextField();

                    jtf2.setText( jtf.getText() );
                }
            }
        }
    }//GEN-LAST:event_jTF_languageToShowGamesPiecesString1FocusLost

    private void jCB_languageToShowGames1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCB_languageToShowGames1ActionPerformed
        // TODO add your handling code here:

        if( ! _modifiedByProgram )
        {
            Object obj = evt.getSource();

            if( obj instanceof JComboBox )
            {
                JComboBox combo = (JComboBox) obj;

                ChessLanguageConfiguration.LanguageConfigurationData lcd =
                (ChessLanguageConfiguration.LanguageConfigurationData) combo.getSelectedItem();

                boolean isLinkedCombo = false;
                _modifiedByProgram = true;
                updateNewComboSelection( combo, lcd, isLinkedCombo );
                _modifiedByProgram = false;
            }
        }
    }//GEN-LAST:event_jCB_languageToShowGames1ActionPerformed

    private void jCb_languageActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCb_languageActionPerformed
        // TODO add your handling code here:

        int selectedIndex = jCb_language.getSelectedIndex();

        if( selectedIndex >= 0 )
        {
            String language = (String) jCb_language.getItemAt( selectedIndex );
            try
            {
                getAppStrConf().changeLanguage( language );
            }
            catch( Throwable th )
            {
                th.printStackTrace();
            }

            try
            {
                ChessStringsConf.instance().changeLanguage( language );
            }
            catch( Throwable th )
            {
                th.printStackTrace();
            }

            try
            {
                if( a_intern != null )
                {
                    a_intern.changeLanguage( language );
                }
            }
            catch( Throwable th )
            {
                th.printStackTrace();
            }

			boolean initialConfiguration = false;
			fillInComboBoxesOfChessLanguage( initialConfiguration );
		}
    }//GEN-LAST:event_jCb_languageActionPerformed

    private void jTf_additionalLanguageKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jTf_additionalLanguageKeyReleased
        // TODO add your handling code here:

//		M_fillLanguageComboBox();
		updateLanguageCombobox();
    }//GEN-LAST:event_jTf_additionalLanguageKeyReleased

    private void jB_addLanguageActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_addLanguageActionPerformed
        // TODO add your handling code here:

		if( M_hasToCreateAdditionalLanguageFolder() )
		{
			M_createFolderAndCopyLanguageConfigurationFilesFromEnglish();
		}

		M_fillLanguageComboBox();

		if( getParent() instanceof MainWindow )
		{
			MainWindow mw = (MainWindow) getParent();
			mw.addLanguageToMenu(jTf_additionalLanguage.getText());
		}
		
    }//GEN-LAST:event_jB_addLanguageActionPerformed

    private void jTf_additionalLanguageFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jTf_additionalLanguageFocusLost
        // TODO add your handling code here:

		updateLanguageCombobox();

    }//GEN-LAST:event_jTf_additionalLanguageFocusLost

	protected void updateNewComboSelection( JComboBox combo,
							ChessLanguageConfiguration.LanguageConfigurationData lcd,
							boolean isLinkedCombo )
	{
		ComboBoxOfChessLanguageData data = _mapComboBoxes.get( combo );

		if( isLinkedCombo )
		{
			combo.setSelectedItem(lcd);
		}

		if( data != null )
		{
			JTextField jtf = data.getLinkedTextField();

			if( lcd != null )
			{
				if(  jtf != null )
				{
					if( lcd == ChessLanguageConfiguration.getCustomLanguage() )
					{
						jtf.setText( "" );
						jtf.setEnabled( true );
						if( !isLinkedCombo )
							jtf.requestFocus( true );
					}
					else
					{
						jtf.setText( lcd._stringOfPieceCodes );
						jtf.setEnabled( false );
					}
				}
			}

			if( !isLinkedCombo && ( data.getLinked() != null ) )
			{
				updateNewComboSelection( data.getLinked(), lcd, true );
			}
		}
	}

	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		/* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
		 * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
		 */
		try {
			for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
				if ("Nimbus".equals(info.getName())) {
					javax.swing.UIManager.setLookAndFeel(info.getClassName());
					break;
				}
			}
		} catch (ClassNotFoundException ex) {
			java.util.logging.Logger.getLogger(JDial_applicationConfiguration.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (InstantiationException ex) {
			java.util.logging.Logger.getLogger(JDial_applicationConfiguration.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (IllegalAccessException ex) {
			java.util.logging.Logger.getLogger(JDial_applicationConfiguration.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (javax.swing.UnsupportedLookAndFeelException ex) {
			java.util.logging.Logger.getLogger(JDial_applicationConfiguration.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
        //</editor-fold>

		/* Create and display the dialog */
		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
/*				JDial_applicationConfiguration dialog = new JDial_applicationConfiguration(new javax.swing.JFrame(), true);
				dialog.addWindowListener(new java.awt.event.WindowAdapter() {
					@Override
					public void windowClosing(java.awt.event.WindowEvent e) {
						System.exit(0);
					}
				});
				dialog.setVisible(true);
*/			}
		});
	}

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jB_addLanguage;
    private javax.swing.JCheckBox jCB_detachedWindowsAlwaysOnTop;
    private javax.swing.JComboBox jCB_languageToParseGamesFrom;
    private javax.swing.JComboBox jCB_languageToShowGames;
    private javax.swing.JComboBox jCB_languageToShowGames1;
    private javax.swing.JCheckBox jCB_showComments;
    private javax.swing.JCheckBox jCB_showNAGs;
    private javax.swing.JComboBox jCb_language;
    private javax.swing.JLabel jL_languageToParseGamesFrom;
    private javax.swing.JLabel jL_languageToShowGames;
    private javax.swing.JLabel jL_languageToShowGames1;
    private javax.swing.JLabel jLbl_additionalLanguage1;
    private javax.swing.JLabel jLbl_additionalLanguageFolder;
    private javax.swing.JLabel jLbl_language;
    private javax.swing.JPanel jP_applicationLanguage;
    private javax.swing.JPanel jP_chessView;
    private javax.swing.JPanel jP_language;
    private javax.swing.JPanel jP_languageForChessGames;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTextField jTF_languageToParseGamesFromPiecesString;
    private javax.swing.JTextField jTF_languageToShowGamesPiecesString;
    private javax.swing.JTextField jTF_languageToShowGamesPiecesString1;
    private javax.swing.JTextPane jTP_additionalLanguageFoder;
    private javax.swing.JTabbedPane jTP_tabbedPane;
    private javax.swing.JTextField jTf_additionalLanguage;
    // End of variables declaration//GEN-END:variables

	protected void updateLanguageCombobox()
	{
		jTP_additionalLanguageFoder.setText( M_getAdditionalLanguageFolder( jTf_additionalLanguage.getText() ) );

		boolean buttonEnabled = ( ( jTf_additionalLanguage.getText().length() > 0 ) &&
									( ArrayFunctions.getFirstIndexOfEquals(_arrayOfLanguages, jTf_additionalLanguage.getText() ) < 0 ) );
		jB_addLanguage.setEnabled(buttonEnabled);
	}
	
	protected void M_fillLanguageComboBox()
	{
		_arrayOfLanguages = GenericFunctions.instance().getObtainAvailableLanguages().getTotalArrayOfAvailableLanguages();

		if( (!a_initializing) && ( jCb_language.getItemCount() == _arrayOfLanguages.length ) )
		{}
		else
		{
			int selectedIndex = -1;
/*
			if( a_initializing )
			{
				String language = getAppliConf().getLanguage();
				if( language != null )
				{
					if( language.equals( "EN" ) )	selectedIndex = 0;
					else if( language.equals( "ES" ) )	selectedIndex = 1;
					else if( language.equals( jTf_additionalLanguage.getText() ) )	selectedIndex = 2;
				}
			}
			else if( jCb_language.getSelectedIndex() > 0 )
			{
				if( jCb_language.getSelectedIndex() == 2 )
				{
					if( ((String) jCb_language.getItemAt(2)).compareTo(jTf_additionalLanguage.getText()) == 0 )
					{
						selectedIndex = 2;
					}
					else selectedIndex = 0;
				}
				else selectedIndex = jCb_language.getSelectedIndex();
			}
*/
			String language = null;
			if( a_initializing )
				language = getAppliConf().getLanguage();
			else
				language = (String) jCb_language.getSelectedItem();

			selectedIndex = ArrayFunctions.getFirstIndexOfEquals(_arrayOfLanguages, language );

//			jCb_language.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "EN", "ES", jTf_additionalLanguage.getText() }));
			jCb_language.setModel(new javax.swing.DefaultComboBoxModel(_arrayOfLanguages));

			if( selectedIndex >= 0 ) jCb_language.setSelectedIndex(selectedIndex);
		}
	}

	protected void M_initializeComponentContents()
	{
		a_initializing = true;

//		jTf_additionalLanguage.setText( getAppliConf().getAdditionalLanguage() );
		M_fillLanguageComboBox();
		jTP_additionalLanguageFoder.setText( M_getAdditionalLanguageFolder( jTf_additionalLanguage.getText() ) );

		jCB_showComments.setSelected( getAppliConf().getHasToShowComments() );
		jCB_showNAGs.setSelected( getAppliConf().getHasToShowNAGs());
		jCB_detachedWindowsAlwaysOnTop.setSelected( getAppliConf().getDetachedGameWindowsAlwaysOnTop() );

		a_initializing=false;
	}

/*
	protected String M_getAdditionalLanguageFolder( String language )
	{
		String result = System.getProperty("user.home") + ConfigurationParent.sa_dirSeparator  + 
						ApplicationConfiguration.sa_MAIN_FOLDER + ConfigurationParent.sa_dirSeparator +
						ApplicationConfiguration.sa_APPLICATION_NAME + ConfigurationParent.sa_dirSeparator +
						ApplicationConfiguration.sa_CONFIGURATION_GROUP + ConfigurationParent.sa_dirSeparator +
						language;
		return( result );
	}
*/	
	protected String M_getAdditionalLanguageFolder( String language )
	{
		return( ( (language==null) || (language.length()==0 ) ) ? "" : getAppliConf().getDefaultLanguageConfigurationFolder( language ) );
	}

	protected boolean M_hasToCreateAdditionalLanguageFolder()
	{
		boolean result = false;
		
		String language = jTf_additionalLanguage.getText();
		if( (language != null) && (! language.equals( "CAT" ) ) )		// el català ja està traduït al .jar
		{
			String folderName = M_getAdditionalLanguageFolder( language );
			File directory = new File( folderName );
			result = ! directory.exists();
		}

		return( result );
	}

	protected void M_createFolderAndCopyLanguageConfigurationFilesFromEnglish()
	{
		String language = jTf_additionalLanguage.getText();
		
		File directory = new File( M_getAdditionalLanguageFolder( language ) );
		boolean ok = directory.mkdirs();

		if( ok )
		{
			M_copyLanguageConfigurationFileFromJarToAdditionalLanguageFolder( "EN", language, "AboutText.rtf" );
			M_copyLanguageConfigurationFileFromJarToAdditionalLanguageFolder( "EN", language, "AppStringsConf.properties" );
			M_copyLanguageConfigurationFileFromJarToAdditionalLanguageFolder( "EN", language, "Application.License.rtf" );
			M_copyLanguageConfigurationFileFromJarToAdditionalLanguageFolder( "EN", language, "DetachedGameWindow_LAN.properties" );
			M_copyLanguageConfigurationFileFromJarToAdditionalLanguageFolder( "EN", language, "EditCommentFrame_LAN.properties" );
			M_copyLanguageConfigurationFileFromJarToAdditionalLanguageFolder( "EN", language, "EditTAGsJFrame_LAN.properties" );
			M_copyLanguageConfigurationFileFromJarToAdditionalLanguageFolder( "EN", language, "InitialPositionDialog_LAN.properties" );
			M_copyLanguageConfigurationFileFromJarToAdditionalLanguageFolder( "EN", language, "JDial_about_LAN.properties" );
			M_copyLanguageConfigurationFileFromJarToAdditionalLanguageFolder( "EN", language, "JDial_applicationConfiguration_LAN.properties" );
			M_copyLanguageConfigurationFileFromJarToAdditionalLanguageFolder( "EN", language, "LicenseJDialog_LAN.properties" );
			M_copyLanguageConfigurationFileFromJarToAdditionalLanguageFolder( "EN", language, "MainWindow_LAN.properties" );
			M_copyLanguageConfigurationFileFromJarToAdditionalLanguageFolder( "EN", language, "PDFScanProgressWindow_LAN.properties" );
			M_copyLanguageConfigurationFileFromJarToAdditionalLanguageFolder( "EN", language, "PDFviewerWindow_LAN.properties" );
			M_copyLanguageConfigurationFileFromJarToAdditionalLanguageFolder( "EN", language, "Splash_LAN.properties" );

			M_copyLanguageConfigurationFileFromJarToAdditionalLanguageFolder( "EN", language, "ChessStringsConf.properties" );
		}
	}

	protected String getChessLanguageForConfiguration( JComboBox combo )
	{
		String result = null;

		if( combo.getSelectedItem() != null )
		{
			ChessLanguageConfiguration.LanguageConfigurationData lcd =
				(ChessLanguageConfiguration.LanguageConfigurationData) combo.getSelectedItem();

			if( lcd == ChessLanguageConfiguration.getCustomLanguage() )
			{
				ComboBoxOfChessLanguageData data = _mapComboBoxes.get( combo );

				if( data != null )
				{
					JTextField jtf = data.getLinkedTextField();

					if(  jtf != null )
					{
						result = jtf.getText();
					}
				}
			}
			else
				result = lcd._languageName;
		}

		return( result );
	}
	
	protected void applyChanges()
	{
		String language = (String) jCb_language.getSelectedItem();

		try
		{
			if( language != null )
				getAppliConf().changeLanguage(language);
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}

/*
		float factor = 0.0F;
		if( jCb_fontSize.getSelectedIndex() == 0 )	factor = SA_FACTOR_NORMAL_FONT_SIZE;
		else if( jCb_fontSize.getSelectedIndex() == 1 )	factor = SA_FACTOR_LARGE_FONT_SIZE;

		if( factor > 0.0F )
		{
			appConf.M_setStrParamConfiguration( ApplicationConfiguration.CONF_APPLICATION_FONT_SIZE, Float.toString(factor) );
		}
*/

		String chessLanguageToParseFrom = getChessLanguageForConfiguration( jCB_languageToParseGamesFrom );
		String chessLanguageToShow = getChessLanguageForConfiguration( jCB_languageToShowGames );

		if( chessLanguageToParseFrom != null )
			getAppliConf().setConfigurationOfChessLanguageToParseTextFrom( chessLanguageToParseFrom );

		if( chessLanguageToShow != null )
			getAppliConf().setConfigurationOfChessLanguageToShow( chessLanguageToShow );

		getAppliConf().setHasToShowComments( jCB_showComments.isSelected() );
		getAppliConf().setHasToShowNAGs( jCB_showNAGs.isSelected() );
		getAppliConf().setDetachedGameWindowsAlwaysOnTop( jCB_detachedWindowsAlwaysOnTop.isSelected() );
	}

	protected void M_copyLanguageConfigurationFileFromJarToAdditionalLanguageFolder( 
							String originLanguage,
							String destinationLanguage,
							String fileName )
	{
		String longFileNameInJar = ApplicationConfiguration.sa_PROPERTIES_PATH_IN_JAR + "/" + originLanguage + "/" + fileName;
		String longFileNameInDisk = getAppliConf().getDefaultLanguageConfigurationFolder( destinationLanguage ) +
									ConfigurationParent.sa_dirSeparator + fileName;

//		M_copyPropertiesFileFromJarToDisk( longFileNameInJar, longFileNameInDisk );

		ResourceFunctions.copyBinaryResourceToFile(longFileNameInJar, longFileNameInDisk );
	}

/*
	protected void M_copyPropertiesFileFromJarToDisk(	String longFileNameInJar,
														String longFileNameInDisk )
	{
		boolean ok = false;
		Properties prop = null;
		InputStream in = null;
		ClassLoader loader = ClassLoader.getSystemClassLoader ();
		in = loader.getResourceAsStream (longFileNameInJar);

		if( in == null )
		{
			try
			{
				in = this.getClass().getClassLoader().getResource(longFileNameInJar).openStream();
			}
			catch( Throwable th )
			{
				in = null;
			}
		}
		

		if (in != null)
		{
			InputStreamReader isr = new InputStreamReader( in, StandardCharsets.UTF_8 );
			if( isr != null )
			{
				prop = new Properties ();
				try
				{
					prop.load (isr); // It can throw IOException
					ok=true;
				}
				catch( Throwable th )
				{
					th.printStackTrace();
				}
				finally
				{
					try
					{
						isr.close();
					}
					catch( Throwable th1 )
					{
						th1.printStackTrace();
					}
				}
			}
		}
		
		if( ok )
		{
			FileOutputStream fos = null;
			OutputStreamWriter osw = null;
			try
			{
				fos = new FileOutputStream( longFileNameInDisk );
				osw = new OutputStreamWriter( fos, StandardCharsets.UTF_8 );
				prop.store( osw, "Default property values to be translated" );
				osw.close();
			}
			catch( Throwable th )
			{
				System.out.println( th.getMessage() );
			}
			finally
			{
				try
				{
					if( osw != null )
						osw.close();
					else if( fos != null )
						fos.close();
				}
				catch( Throwable th )
				{
					th.printStackTrace();
				}
			}
		}
	}
*/
	protected void M_saveInternationalization()
	{
		try
		{
			if( a_intern != null ) a_intern.saveConfiguration();
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}
	}

	public void validateChessLanguageComboBoxChanges( JComboBox combo ) throws ValidateException
	{
		if( combo.getSelectedItem() != null )
		{
			ChessLanguageConfiguration.LanguageConfigurationData lcd =
				(ChessLanguageConfiguration.LanguageConfigurationData) combo.getSelectedItem();

			if( lcd == ChessLanguageConfiguration.getCustomLanguage() )
			{
				ComboBoxOfChessLanguageData data = _mapComboBoxes.get( combo );

				if( data != null )
				{
					JTextField jtf = data.getLinkedTextField();

					if(  jtf != null )
					{
						ChessLanguageConfiguration clc = ChessLanguageConfiguration.getConfiguration( jtf.getText() );
						
						if( clc == null )
						{
							try
							{
								// if the data to create a new language configuration is not valid, it will throw an exception.
								ChessLanguageConfiguration.createConfiguration(null, jtf.getText());
							}
							catch( Throwable th )
							{
								th.printStackTrace();
								throw( new ValidateException( th.getMessage(), jtf ) );
							}
						}
					}
				}
			}
		}
	}

	public void validateChanges() throws ValidateException
	{
		validateChessLanguageComboBoxChanges( jCB_languageToParseGamesFrom );
		validateChessLanguageComboBoxChanges( jCB_languageToShowGames );
	}

	@Override
	public void accept(InformerInterface panel)
	{
		boolean proceed = false;
		try
		{
			validateChanges();
			proceed = true;
		}
		catch( ValidateException ve )
		{
			GenericFunctions.instance().getDialogsWrapper().showMessageDialog( this, ve.getMessage(),
											getAppStrConf().getProperty( AppStringsConf.CONF_CONFIGURATION_ERROR ),
											DialogsWrapper.ERROR_MESSAGE );

			Component comp = ve.getComponentNotValidated();
			if( comp != null )
			{
				boolean showFocus = true;
				a_intern.setFocus( comp, showFocus );
			}
		}

		if( proceed )
		{
			applyChanges();

			try
			{
				ApplicationConfiguration.instance().M_saveConfiguration();
			}
			catch( Throwable th )
			{
				th.printStackTrace();
			}
/*
			if( M_hasToCreateAdditionalLanguageFolder() )
			{
				M_createFolderAndCopyLanguageConfigurationFilesFromEnglish();
			}
*/
			M_saveInternationalization();
			a_userHasPressedOK = true;
			setVisible(false);
		}
	}

	@Override
	public void cancel(InformerInterface panel)
	{
		// we load again the Strings configuration with the language which with was called the dialogue
		try
		{
			AppStringsConf.instance().changeLanguage( a_languageInConstructor );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}

		try
		{
			ChessStringsConf.instance().changeLanguage( a_languageInConstructor );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}

		M_saveInternationalization();
        setVisible(false);
	}

	@Override
	public void revert(InformerInterface panel)
	{
		try
		{
			a_intern.changeLanguage( getAppliConf().getLanguage() );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}

		M_initializeComponentContents();

		boolean initialConfiguration = true;
		fillInComboBoxesOfChessLanguage( initialConfiguration );
	}

	public boolean getUserHasPressedOK()
	{
		return( a_userHasPressedOK );
	}

	public void releaseResources()
	{
		a_intern=null;	// for the garbage collector to free the memory of the internationallization object and after the memory of this form
	}

	protected void fillInComboBoxesOfChessLanguage( boolean initialConfiguration )
	{
		if( _mapComboBoxes == null )
		{
			_mapComboBoxes = new HashMap<Component, ComboBoxOfChessLanguageData>();
			_mapComboBoxes.put( jCB_languageToParseGamesFrom,
							new ComboBoxOfChessLanguageData( jCB_languageToParseGamesFrom,
															null,
															jTF_languageToParseGamesFromPiecesString ) );
			_mapComboBoxes.put( jCB_languageToShowGames,
							new ComboBoxOfChessLanguageData( jCB_languageToShowGames,
															jCB_languageToShowGames1,
															jTF_languageToShowGamesPiecesString ) );
			_mapComboBoxes.put( jCB_languageToShowGames1,
							new ComboBoxOfChessLanguageData( jCB_languageToShowGames1,
															jCB_languageToShowGames,
															jTF_languageToShowGamesPiecesString1 ) );
			_mapComboBoxes.put( jTF_languageToShowGamesPiecesString,
							new ComboBoxOfChessLanguageData( jCB_languageToShowGames,
															jCB_languageToShowGames1,
															jTF_languageToShowGamesPiecesString ) );
			_mapComboBoxes.put( jTF_languageToShowGamesPiecesString1,
							new ComboBoxOfChessLanguageData( jCB_languageToShowGames1,
															jCB_languageToShowGames,
															jTF_languageToShowGamesPiecesString1 ) );
		}

		Vector< ChessLanguageConfiguration.LanguageConfigurationData > vector = ChessLanguageConfiguration.getListOfLanguages();

		fillInOneComboBoxOfChessLanguage( jCB_languageToParseGamesFrom, vector,
											( initialConfiguration ?
												getAppliConf().getConfigurationOfChessLanguageToParseTextFrom() :
												null ) );
		fillInOneComboBoxOfChessLanguage( jCB_languageToShowGames, vector,
											( initialConfiguration ?
												getAppliConf().getConfigurationOfChessLanguageToShow() :
												null ) );
		fillInOneComboBoxOfChessLanguage( jCB_languageToShowGames1, vector,
											( initialConfiguration ?
												getAppliConf().getConfigurationOfChessLanguageToShow() :
												null ) );
	}

	protected void fillInOneComboBoxOfChessLanguage( JComboBox combo,
													Vector< ChessLanguageConfiguration.LanguageConfigurationData > vector,
													String defaultValue )
	{
		ChessLanguageConfiguration.LanguageConfigurationData selectedItem = null;
		if( defaultValue != null )
		{
			selectedItem = ChessLanguageConfiguration.getLanguageConfigurationData( defaultValue );
		}
		else if( ( combo.getModel() != null ) && ( combo.getModel().getSize() > 0 ) )
		{
			selectedItem = ( ChessLanguageConfiguration.LanguageConfigurationData ) combo.getSelectedItem();
		}
		DefaultComboBoxModel<ChessLanguageConfiguration.LanguageConfigurationData> dcbm =
			new DefaultComboBoxModel<ChessLanguageConfiguration.LanguageConfigurationData>( vector );
		combo.setModel( dcbm );
		combo.setSelectedItem( selectedItem );
	}


	protected static class ComboBoxOfChessLanguageData
	{
		protected JComboBox _own = null;
		protected JComboBox _linked = null;
		protected JTextField _linkedTextField = null;

		public ComboBoxOfChessLanguageData( JComboBox own, JComboBox linked, JTextField linkedTextField )
		{
			_own = own;
			_linked = linked;
			_linkedTextField = linkedTextField;
		}
		
		public JComboBox getOwn()
		{
			return( _own );
		}
		
		public JComboBox getLinked()
		{
			return( _linked );
		}
		
		public JTextField getLinkedTextField()
		{
			return( _linkedTextField );
		}
	}
	
	protected AppStringsConf getAppStrConf()
	{
		return( AppStringsConf.instance() );
	}

	@Override
	public ApplicationConfiguration getAppliConf()
	{
		return( (ApplicationConfiguration) super.getAppliConf() );
	}

}
