/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/gpl-3.0.txt
 *
 */
package com.frojasg1.chesspdfbrowser.application;

import com.frojasg1.chesspdfbrowser.configuration.AppStringsConf;
import com.frojasg1.chesspdfbrowser.configuration.ApplicationConfiguration;
import com.frojasg1.chesspdfbrowser.interfaces.PDFownerInterface;
import com.frojasg1.chesspdfbrowser.pdf.renderer.MyPDFRenderer;
import com.frojasg1.chesspdfbrowser.pdf.textsegmentation.PageSegmentationResult;
import com.frojasg1.chesspdfbrowser.pdf.textsegmentation.PixelComponents;
import com.frojasg1.chesspdfbrowser.pdf.textsegmentation.implementation.PDFPageSegmentator;
import com.frojasg1.chesspdfbrowser.pdf.textsegmentation.implementation.SummarizedPage;
import com.frojasg1.chesspdfbrowser.threads.LoadControllerInterface;
import com.frojasg1.general.desktop.view.panels.NavigatorControllerInterface;
import com.frojasg1.general.desktop.view.panels.NavigatorJPanel;
import com.frojasg1.chesspdfbrowser.view.chess.interaction.ChessGameControllerInterface;
import com.frojasg1.general.desktop.ImageFunctions;
import com.frojasg1.general.desktop.view.panels.InformerInterface;
import com.frojasg1.general.desktop.view.pdf.ImageJPanel;
import com.frojasg1.general.desktop.view.pdf.ImageJPanelControllerInterface;
import com.hotmail.frojasg1.applications.common.components.internationalization.window.InternationalizedJFrame;
import com.hotmail.frojasg1.applications.common.components.resizecomp.MapResizeRelocateComponentItem;
import com.hotmail.frojasg1.applications.common.components.resizecomp.ResizeRelocateItem;
import com.frojasg1.general.dialogs.DialogsWrapper;
import com.frojasg1.general.number.DoubleReference;
import com.frojasg1.general.number.IntegerFunctions;
import com.frojasg1.generic.GenericFunctions;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import javax.imageio.ImageIO;
import javax.swing.DefaultComboBoxModel;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.rendering.PDFRenderer;

// Minimum Size: [570, 780]

/**
 *
 * @author Usuario
 */
public class PDFviewerWindow extends InternationalizedJFrame
							implements NavigatorControllerInterface,
										ImageJPanelControllerInterface,
										PDFownerInterface
{
	public static final String sa_configurationBaseFileName = "PDFviewerWindow";

	protected NavigatorJPanel _navigatorPanel = null;
	protected ImageJPanel _pdfImagePanel = null;

	protected PDFownerInterface _pdfOwner = null;

	protected LoadControllerInterface _parent = null;
	protected ChessGameControllerInterface _chessGameController = null;

	protected int _totalNumberOfPages = 0;
	protected int _currentPage = 0;

	protected PDDocument _pdfDocument = null;
	protected PDFRenderer _pdfRenderer = null;

	protected PDFPageSegmentator _pageSegmentator = null;

	protected boolean _putSummarizedImage = false;
	protected static final float _dpi_of_pdf_page_to_show_factor_1 = 72F;
//	protected float _dpi_of_pdf_page_to_show = 72F;

	protected boolean _flagComboBoxItemSelectedByProgram = false;
	
	/**
	 * Creates new form MainWindow
	 */
	public PDFviewerWindow( LoadControllerInterface parent,
							ChessGameControllerInterface cgci )
	{
		super( ApplicationConfiguration.instance() );

		_chessGameController = cgci;
		_pdfOwner = this;
		_parent = parent;

		initComponents();

		initPanel();

		setWindowConfiguration( );

//		setImage( "L:\\USB\\K\\Ajedrez\\Libros.2016-02\\salida.pdfbox.el peon pasado\\custom-render-0.png" );
	}
	
	public void setNewPDF( String fileName, PDDocument document )
	{
		setTitle( fileName );
		setDocument( document );
		_pdfRenderer = new MyPDFRenderer( _pdfDocument );
		setNumberOfPages( _pdfDocument.getNumberOfPages() );
		setRenderSegmentsOfPages( hasToShowSegments() );
	}

	public boolean hasToShowSegments()
	{
		return( getAppliConf().getHasToShowSegments() );
	}

	protected void setDocument( PDDocument document )
	{
		if( _pdfDocument != null )
		{
			try
			{
				_pdfDocument.close();
			}
			catch( Throwable th )
			{
				th.printStackTrace();
			}
		}

		_pdfDocument = document;
	}

	protected void setImage( String fileName )
	{
		try
		{
			File file = new File( fileName );
			setPage( ImageIO.read( file ) );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}
	}

	public void setNumberOfPages( int numberOfPages )
	{
		_totalNumberOfPages = numberOfPages;
		_currentPage = 0;

		updatePage( 0 );
	}

	public void updateCurrentPageTexts()
	{
		jTF_currentPage.setText( String.valueOf( _currentPage + 1 ) );
		jTFTotalPages.setText( String.valueOf( _totalNumberOfPages ) );
	}
	
	public void updatePage( int pageIndex )
	{
		setPage(_pdfOwner.getPage( pageIndex, getSelectedZoomFactorDouble() ), getSelectedZoomFactor(), pageIndex );
		updateCurrentPageTexts();
	}

	protected void initPanel()
	{
		try
		{
			_navigatorPanel = new NavigatorJPanel( this );
			_navigatorPanel.setBounds(0, 0, 220, 50);
			_navigatorPanel.setBorder( jPanel3.getBorder() );
			jPanel3.add( _navigatorPanel );

			_pdfImagePanel = new ImageJPanel( jSP_imageScrollPane, this );
			jSP_imageScrollPane.setViewportView( _pdfImagePanel );

			pack();
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}
	}

	protected void setWindowConfiguration( )
	{
		MapResizeRelocateComponentItem mapRRCI = new MapResizeRelocateComponentItem();
		try
		{
			mapRRCI.putResizeRelocateComponentItem( jPanel4, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.RESIZE_TO_BOTTOM );
			mapRRCI.putResizeRelocateComponentItem( jPanel1, ResizeRelocateItem.RESIZE_TO_RIGHT );
			mapRRCI.putResizeRelocateComponentItem( jPanel2, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.RESIZE_TO_BOTTOM );
			mapRRCI.putResizeRelocateComponentItem( jSP_imageScrollPane, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.RESIZE_TO_BOTTOM );
			mapRRCI.putResizeRelocateComponentItem( _pdfImagePanel, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.RESIZE_TO_BOTTOM );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}
/*
		a_intern = new JFrameInternationalization(	ApplicationConfiguration.sa_MAIN_FOLDER,
													ApplicationConfiguration.sa_APPLICATION_NAME,
													ApplicationConfiguration.sa_CONFIGURATION_GROUP,
													ApplicationConfiguration.sa_PROPERTIES_PATH_IN_JAR,
													a_configurationBaseFileName,
													this,
													parent,
													a_vectorJpopupMenus,
													false,
													mapRRCI );
*/
		createInternationalization(	getAppliConf().getConfigurationMainFolder(),
									getAppliConf().getApplicationNameFolder(),
									getAppliConf().getApplicationGroup(),
									getAppliConf().getInternationalPropertiesPathInJar(),
									sa_configurationBaseFileName,
									this,
									null,
									null,
									true,
									mapRRCI );

//		setIcon( "com/frojasg1/chesspdfbrowser/resources/icons/App.icon.png" );
	}
	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel4 = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        jLZoomFactor = new javax.swing.JLabel();
        jCBZoomFactor = new javax.swing.JComboBox();
        jLPage = new javax.swing.JLabel();
        jTF_currentPage = new javax.swing.JTextField();
        jLabel1 = new javax.swing.JLabel();
        jTFTotalPages = new javax.swing.JTextField();
        jPanel3 = new javax.swing.JPanel();
        jB_scanPDFforGames = new javax.swing.JButton();
        jPanel2 = new javax.swing.JPanel();
        jSP_imageScrollPane = new javax.swing.JScrollPane();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setMinimumSize(new java.awt.Dimension(570, 340));
        setName("PDFViewerWindow"); // NOI18N
        setPreferredSize(new java.awt.Dimension(570, 780));
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });
        getContentPane().setLayout(null);

        jPanel4.setMinimumSize(new java.awt.Dimension(552, 300));
        jPanel4.setName("jPanel4"); // NOI18N
        jPanel4.setLayout(null);

        jPanel1.setName("jPanel1"); // NOI18N
        jPanel1.setPreferredSize(new java.awt.Dimension(510, 50));
        jPanel1.setLayout(null);

        jLZoomFactor.setText("Zoom factor:");
        jLZoomFactor.setName("jLZoomFactor"); // NOI18N
        jPanel1.add(jLZoomFactor);
        jLZoomFactor.setBounds(10, 10, 110, 14);

        jCBZoomFactor.setName("jCBZoomFactor"); // NOI18N
        jCBZoomFactor.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCBZoomFactorActionPerformed(evt);
            }
        });
        jPanel1.add(jCBZoomFactor);
        jCBZoomFactor.setBounds(10, 30, 90, 20);

        jLPage.setText("Page:");
        jLPage.setName("jLPage"); // NOI18N
        jPanel1.add(jLPage);
        jLPage.setBounds(360, 40, 70, 14);

        jTF_currentPage.setText("0");
        jTF_currentPage.setName(""); // NOI18N
        jTF_currentPage.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTF_currentPageActionPerformed(evt);
            }
        });
        jPanel1.add(jTF_currentPage);
        jTF_currentPage.setBounds(430, 40, 30, 20);

        jLabel1.setFont(new java.awt.Font("Tahoma", 0, 18)); // NOI18N
        jLabel1.setText("/");
        jPanel1.add(jLabel1);
        jLabel1.setBounds(460, 40, 10, 20);

        jTFTotalPages.setEditable(false);
        jTFTotalPages.setEnabled(false);
        jTFTotalPages.setFocusable(false);
        jTFTotalPages.setName(""); // NOI18N
        jPanel1.add(jTFTotalPages);
        jTFTotalPages.setBounds(470, 40, 30, 20);

        jPanel3.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 218, Short.MAX_VALUE)
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 48, Short.MAX_VALUE)
        );

        jPanel1.add(jPanel3);
        jPanel3.setBounds(130, 10, 220, 50);

        jB_scanPDFforGames.setText("Scan PDF for games");
        jB_scanPDFforGames.setName("jB_scanPDFforGames"); // NOI18N
        jB_scanPDFforGames.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jB_scanPDFforGamesActionPerformed(evt);
            }
        });
        jPanel1.add(jB_scanPDFforGames);
        jB_scanPDFforGames.setBounds(360, 10, 180, 23);

        jPanel4.add(jPanel1);
        jPanel1.setBounds(0, 0, 550, 70);

        jPanel2.setName("jPanel2"); // NOI18N
        jPanel2.setPreferredSize(new java.awt.Dimension(510, 250));
        jPanel2.setLayout(null);
        jPanel2.add(jSP_imageScrollPane);
        jSP_imageScrollPane.setBounds(0, 0, 550, 670);

        jPanel4.add(jPanel2);
        jPanel2.setBounds(0, 70, 550, 670);

        getContentPane().add(jPanel4);
        jPanel4.setBounds(0, 0, 553, 740);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        // TODO add your handling code here:

		closeWindow();

    }//GEN-LAST:event_formWindowClosing

    private void jCBZoomFactorActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCBZoomFactorActionPerformed
        // TODO add your handling code here:

		if( ! _flagComboBoxItemSelectedByProgram )
		{
			DoubleReference newFactor = getSelectedZoomFactor();
			setNewZoomFactor( newFactor );
		}
    }//GEN-LAST:event_jCBZoomFactorActionPerformed

    private void jTF_currentPageActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTF_currentPageActionPerformed
        // TODO add your handling code here:

		BufferedImage image = null;
		int numberOfPage = -1;
		try
		{
			numberOfPage = Integer.parseInt( jTF_currentPage.getText() );
		}
		catch( Throwable th )
		{}

		if( numberOfPage > -1 )
		{
			image = _pdfOwner.getPage( numberOfPage - 1, getSelectedZoomFactorDouble() );
			if( image != null )
			{
				setPage( image, getSelectedZoomFactor(), numberOfPage - 1 );
			}
		}
/*
		if( image != null )
		{
			_currentPage = numberOfPage - 1;
		}
		*/
//		updateComboBoxValues();

    }//GEN-LAST:event_jTF_currentPageActionPerformed

    private void jB_scanPDFforGamesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_scanPDFforGamesActionPerformed
        // TODO add your handling code here:

		if( _chessGameController.checkToSaveCurrentPGN() )
			scanPDFDocumentForGames();

    }//GEN-LAST:event_jB_scanPDFforGamesActionPerformed

	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		/* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
		 * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
		 */
/*
		try {
			for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
				if ("Nimbus".equals(info.getName())) {
					javax.swing.UIManager.setLookAndFeel(info.getClassName());
					break;
				}
			}
		} catch (ClassNotFoundException ex) {
			java.util.logging.Logger.getLogger(PDFviewerWindow.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (InstantiationException ex) {
			java.util.logging.Logger.getLogger(PDFviewerWindow.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (IllegalAccessException ex) {
			java.util.logging.Logger.getLogger(PDFviewerWindow.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (javax.swing.UnsupportedLookAndFeelException ex) {
			java.util.logging.Logger.getLogger(PDFviewerWindow.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
        //</editor-fold>
        //</editor-fold>
*/
		/* Create and display the form */
		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
//				PDFviewerWindow window = new PDFviewerWindow();
//				window.setVisible(true);
			}
		});
	}

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jB_scanPDFforGames;
    private javax.swing.JComboBox jCBZoomFactor;
    private javax.swing.JLabel jLPage;
    private javax.swing.JLabel jLZoomFactor;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JScrollPane jSP_imageScrollPane;
    private javax.swing.JTextField jTFTotalPages;
    private javax.swing.JTextField jTF_currentPage;
    // End of variables declaration//GEN-END:variables

	protected DoubleReference getSelectedZoomFactor()
	{
        DoubleReference newFactor = null;
        Object obj = jCBZoomFactor.getSelectedItem();
        if( obj instanceof DoubleReference )
        {
            newFactor = (DoubleReference) obj;
        }
		return( newFactor );
	}

	protected double getSelectedZoomFactorDouble()
	{
		double factor = 1.0D;
		
		DoubleReference dr = getSelectedZoomFactor();
		if( dr != null )
			factor = dr._value;
		
		return( factor );
	}

	protected float getDPI( double factor )
	{
		float result = (float) (_dpi_of_pdf_page_to_show_factor_1 * factor );
		return( result );
	}
	
	protected void setPage( BufferedImage image, DoubleReference factor, int pageIndex )
	{
		ImageJPanel.InitialVerticalPosition ivp = null;
				
		if( _currentPage < pageIndex )
			ivp = ImageJPanel.InitialVerticalPosition.TOP;
		else if( _currentPage > pageIndex )
			ivp = ImageJPanel.InitialVerticalPosition.BOTTOM;
		else
			ivp = ImageJPanel.InitialVerticalPosition.NOTHING;
				
		_pdfImagePanel.setImage( image, factor, ivp );
		updateComboBoxValues();

		_flagComboBoxItemSelectedByProgram = true;
		jCBZoomFactor.setSelectedItem( _pdfImagePanel.getZoomFactor() );
		_flagComboBoxItemSelectedByProgram = false;
		
		_currentPage = pageIndex;
	}

	protected void setPage( BufferedImage image )
	{
		DoubleReference factor = new DoubleReference( 1.0D );
		setPage( image, factor, _currentPage );
	}

	protected void updateComboBoxValues()
	{
		List<DoubleReference> listOfZoomFactors = _pdfImagePanel.getStepsForFactorList();
		if( listOfZoomFactors != null )
		{
			DefaultComboBoxModel<DoubleReference> dcbm = new DefaultComboBoxModel<DoubleReference>( new Vector<DoubleReference>( listOfZoomFactors ) );
			jCBZoomFactor.setModel( dcbm );
		}
	}

	@Override
	public void navigator_start( InformerInterface panel )
	{
		int pageIndex = 0;
		updatePage( pageIndex );
	}

	@Override
	public void navigator_end( InformerInterface panel )
	{
		int pageIndex = _totalNumberOfPages - 1;
		updatePage( pageIndex );
	}

	protected int incrementCurrentPage()
	{
		int pageIndex = IntegerFunctions.min( _totalNumberOfPages - 1, _currentPage + 1 );
//		_currentPage = IntegerFunctions.min( _totalNumberOfPages - 1, _currentPage + 1 );
		return( pageIndex );
	}
	
	protected int decrementCurrentPage()
	{
		int pageIndex = IntegerFunctions.max( 0, _currentPage - 1 );
//		_currentPage = IntegerFunctions.max( 0, _currentPage - 1 );
		return( pageIndex );
	}

	@Override
	public void navigator_previous( InformerInterface panel )
	{
		updatePage( decrementCurrentPage() );
	}

	@Override
	public void navigator_next( InformerInterface panel )
	{
		updatePage( incrementCurrentPage() );
	}

	protected int getCurrentPage()
	{
		return( _currentPage );
	}

	@Override
	public void previousPage()
	{
		navigator_previous( null );
	}

	@Override
	public void nextPage()
	{
		navigator_next( null );
	}

	@Override
	public void setNewZoomFactor( DoubleReference factor )
	{
		setPage(_pdfOwner.getPage( _currentPage, factor._value ), factor, _currentPage );
	}
/*
	protected void factorUpdatePdfImagePanel( DoubleReference factor )
	{
        if( factor != null )
        {
			_pdfImagePanel.setImage( getPage( _currentPage, factor._value ), factor );
        }
	}
*/	
	
	@Override
	public BufferedImage getPage( int numberOfPage, double factor )
	{
		BufferedImage image = null;

		try
		{
			float dpi = getDPI( factor );
			
			image = _pdfRenderer.renderImageWithDPI( numberOfPage, dpi );

			if( _putSummarizedImage )
			{
				Dimension size = new Dimension( (int) image.getWidth(),
												(int) image.getHeight() );

				BufferedImage tmpImage = createImageFromSummarizedPage( _pageSegmentator.getSummarizedPage(size) );

				if( tmpImage != null )	image = tmpImage;

				_putSummarizedImage = false;
			}
			else if( hasToShowSegments() )
			{
				PageSegmentationResult result = _pageSegmentator.getSegments( numberOfPage );

//				if( result.getCouldBeValidated() )
				{
					image = renderSegments( image, result.getIteratorToSegmentedRegions(), dpi / 72F );
				}
			}
		}
		catch( Throwable th )
		{
			th.printStackTrace();
			GenericFunctions.instance().getDialogsWrapper().showMessageDialog( this,
											getAppStrConf().getProperty( AppStringsConf.CONF_ERROR_LOADING_PAGE_FROM ) +
											getTitle() +
											". " + getAppStrConf().getProperty( AppStringsConf.CONF_ERROR ) + ": " +
											th.getMessage(),
											getAppStrConf().getProperty( AppStringsConf.CONF_ERROR_LOADING_PAGE ),
											DialogsWrapper.ERROR_MESSAGE );
		}

		return( image );
	}

	protected BufferedImage renderSegments( BufferedImage image, Iterator<Rectangle> it, double factor )
	{
/*		BufferedImage result = new BufferedImage( (int) image.getWidth(),
													(int) image.getHeight(),
													BufferedImage.TYPE_INT_ARGB );
*/
		BufferedImage result = image;
		Graphics2D grph = result.createGraphics();

//		grph.drawImage( image, null, null );

		int index = 1;
		while( it.hasNext() )
		{
			Rectangle rect = it.next();

			rect.setRect( rect.getX() * factor,
							rect.getY() * factor,
							rect.getWidth() * factor,
							rect.getHeight() * factor );

			renderSegment( grph, rect, index );
			index++;
		}
		
		return( result );
	}

	protected void renderSegment( Graphics grph, Rectangle rect, int index )
	{
		grph.setColor( Color.RED );

		int xx = (int) rect.getX()-3;
		int yy = (int) rect.getY()-3;
		int width = (int) rect.getWidth()+6;
		int height = (int) rect.getHeight()+6;
		for( int ii=0; ii<7; ii++ )
		{
			grph.drawRect( xx, yy, width, height );
			xx++;
			yy++;
			width -= 2;
			height -= 2;
		}

//		Integer fontSize = IntegerFunctions.parseInt( jTF_fontSize.getText() );
		Integer fontSize = (int) getDPI( 1.0D );
		if( fontSize == null ) fontSize = 72;
		Font font = new Font( "Lucida", Font.BOLD, IntegerFunctions.min( fontSize, IntegerFunctions.min( (int) rect.getWidth(), (int) rect.getHeight() ) ) );
		grph.setFont(font);
		ImageFunctions.paintStringCentered( grph, font, String.valueOf( index ), Color.RED, rect, null );
	}

    protected void setRenderSegmentsOfPages( boolean hasToSetSegments )
	{
		if( hasToSetSegments && ( _pdfDocument != null ) )
		{
			if( _pageSegmentator == null )
				_pageSegmentator = new PDFPageSegmentator( getAppliConf() );

			_pageSegmentator.setChessViewConfiguration( getAppliConf() );

			if( _pageSegmentator.getPDDocument() != _pdfDocument )
			{
				_parent.block();

				try
				{
					_pageSegmentator.initialize( _pdfDocument, 3, 30, null );
				}
				catch( Throwable th )
				{
					th.printStackTrace();

					_pageSegmentator = new PDFPageSegmentator( getAppliConf() );
//					jCB_renderSegments.setSelected( false );

					GenericFunctions.instance().getDialogsWrapper().showMessageDialog( this,
													getAppStrConf().getProperty( AppStringsConf.CONF_ERROR_INITIALIZING_SEGMENTATOR ) + th,
													getAppStrConf().getProperty( AppStringsConf.CONF_ERROR_INITIALIZING_SEGMENTATOR2 ),
													DialogsWrapper.ERROR_MESSAGE );
				}

				_parent.unblock();
			}
			_putSummarizedImage = true;
		}

		updatePage( _currentPage );
    }                                                  

	protected BufferedImage createImageFromSummarizedPage( SummarizedPage sumPage )
	{
		PixelComponents[][] pixels = null;

		BufferedImage result = null;
		
		if( sumPage != null )
		{
			pixels = sumPage.getSummarizedPixels();

			result = new BufferedImage( pixels.length, pixels[0].length, BufferedImage.TYPE_INT_RGB );

			for( int xx=0; xx<pixels.length; xx++ )
				for( int yy=0; yy<pixels[xx].length; yy++ )
				{
					PixelComponents pixel = pixels[xx][yy];
					int pixelValue = 0;
					if( pixel != null )
					{
						pixelValue = ( ( (int) pixel.getRed() ) << 16 ) |
									( ( (int) pixel.getGreen() ) << 8 ) |
									( (int) pixel.getBlue() );
					}
					result.setRGB( xx, yy, pixelValue );
				}
		}

		return( result );
	}

	public void block()
	{
		changeToWaitCursor();
	}

	public void unblock()
	{
		revertChangeToWaitCursor();
	}

	@Override
	public ApplicationConfiguration getAppliConf()
	{
		ApplicationConfiguration result = null;
		if( _appliConf instanceof ApplicationConfiguration )
			result = (ApplicationConfiguration) _appliConf;

		return( result );
	}

	protected void scanPDFDocumentForGames()
	{
		PDFScanProgressJDial scanProgressJDial = new PDFScanProgressJDial( this, true, _parent, _pdfDocument );

//		PDFScanProgressWindow scanProgressJFrame = new PDFScanProgressWindow( this, _parent, _pdfDocument );

		scanProgressJDial.setAlwaysOnTop(true);
		scanProgressJDial.setVisible( true );
	}

	public void closeWindow()
	{
		boolean closeWindow = false;
		formWindowClosing( closeWindow );
		setVisible(false);
	}

	public AppStringsConf getAppStrConf()
	{
		return( AppStringsConf.instance() );
	}
}
