/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/gpl-3.0.txt
 *
 */
package com.frojasg1.chesspdfbrowser.threads;

import com.frojasg1.chesspdfbrowser.configuration.ApplicationConfiguration;
import com.frojasg1.chesspdfbrowser.io.file.implementation.PgnChessFile;
import com.frojasg1.chesspdfbrowser.model.chess.game.ChessGame;
import com.frojasg1.chesspdfbrowser.view.chess.interaction.ChessGameControllerInterface;
import java.awt.Rectangle;
import java.io.File;
import java.util.List;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.pdmodel.PDPage;
import org.apache.pdfbox.text.PDFTextStripperByArea;

/**
 *
 * @author Usuario
 */
public class LoadPDFThread extends Thread
{
	protected LoadControllerInterface _controller = null;
	protected String _fileName = null;
	protected PDDocument _pdfDocument = null;

	public LoadPDFThread( LoadControllerInterface controller, String fileName )
	{
		_controller = controller;
		_fileName = fileName;
	}

	@Override
	public void run()
	{
		try
		{
			if( _controller != null )	_controller.block();

			_pdfDocument = PDDocument.load( new File(_fileName) );

			if( _controller != null )
				_controller.newChessPDFLoaded( _pdfDocument );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
			if( _controller != null )
			{
				_controller.showError( "Error loading PDF file " + _fileName + ". ERROR: " + th.getMessage(), "Error opening file" );
			}
		}
		finally
		{
/*
			if( _pdfDocument != null )
			{
				try
				{
					_pdfDocument.close();
				}
				catch( Throwable th )
				{
					th.printStackTrace();
				}
			}
*/
			if( _controller != null )
				_controller.unblock();
		}
	}

	public PDDocument getPDFDocument()
	{
		return( _pdfDocument );
	}
}
