/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.chesspdfbrowser.configuration;

import com.frojasg1.chesspdfbrowser.io.notation.ChessMoveNotation;
import com.frojasg1.chesspdfbrowser.io.parsers.movematcher.ChessMoveMatcher;
import com.frojasg1.chesspdfbrowser.model.chess.configuration.ChessLanguageConfiguration;
import com.frojasg1.chesspdfbrowser.model.chess.configuration.ChessViewConfiguration;
import com.frojasg1.chesspdfbrowser.model.chess.configuration.EnglishChessConfiguration;
import com.frojasg1.chesspdfbrowser.view.chess.editcomment.EditCommentConfiguration;
import com.frojasg1.chesspdfbrowser.view.chess.edittags.FilterTAGconfiguration;
import com.frojasg1.applications.common.configuration.application.BaseApplicationConfiguration;
import java.io.File;
import java.util.Properties;

/**
 *
 * @author Usuario
 */
public class ApplicationConfiguration	extends BaseApplicationConfiguration
										implements ChessViewConfiguration, EditCommentConfiguration,
													FilterTAGconfiguration
{
	ChessMoveNotation _chessMoveNotationForView = null;
	ChessMoveMatcher _chessMoveMatcherForView = null;

	public static final String VERSION="v1.0";

	protected static final String RESOURCE_FOR_APPLICATION_ICON="com/frojasg1/chesspdfbrowser/resources/icons/App.icon.png";

	public static final String CONF_LAST_DIRECTORY = "LAST_DIRECTORY";

//	protected static final String APPLICATION_GROUP = "general";
	protected static final String GLOBAL_CONF_FILE_NAME = "GlobalConfiguration.properties";
/*
	protected static final String sa_PROPERTIES_PATH_IN_JAR =	"com" + ConfigurationParent.sa_dirSeparator + 
																"hotmail" + ConfigurationParent.sa_dirSeparator +
																"frojasg1" + ConfigurationParent.sa_dirSeparator + 
																"aplicaciones" + ConfigurationParent.sa_dirSeparator +
																"fileencoderapplication" + ConfigurationParent.sa_dirSeparator +
																"view" + ConfigurationParent.sa_dirSeparator +
																"internationalization" + ConfigurationParent.sa_dirSeparator +
																"properties";
*/
//	public static final String sa_PROPERTIES_PATH_IN_JAR="com/hotmail/frojasg1/applications/fileencoderapplication/view/internationalization/properties";
	public static final String sa_MAIN_FOLDER = ".frojasg1.apps";
	public static String sa_APPLICATION_NAME = null; //"ChessPDFBrowser";
	public static final String sa_CONFIGURATION_GROUP = "Configuration";
//	public static final String sa_PROPERTIES_PATH_IN_JAR="com/frojasg1/applications/chesspdfbrowser/internationalization/properties";
	public static final String sa_PROPERTIES_PATH_IN_JAR="com/frojasg1/app/chesspdfbrowser/inter";

	public static final String ES_LANGUAGE = "ES";	 // Spanish language
	public static final String EN_LANGUAGE = "EN";	 // English language

	public static final String CONF_LANGUAGE = "LANGUAGE";
	public static final String CONF_ADDITIONAL_LANGUAGE = "ADDITIONAL_LANGUAGE";
	public static final String CONF_HAS_TO_SHOW_COMMENTS = "HAS_TO_SHOW_COMMENTS";
	public static final String CONF_HAS_TO_SHOW_SEGMENTS = "HAS_TO_SHOW_SEGMENTS";
	public static final String CONF_EDIT_COMMENT_WINDOW_ALWAYS_ON_TOP = "EDIT_COMMENT_WINDOW_ALWAYS_ON_TOP";

	public static final String CONF_EDIT_TAG_MANDATORY_FILTER_SELECTED = "EDIT_TAG_MANDATORY_FILTER_SELECTED";
	public static final String CONF_EDIT_TAG_PLAYER_FILTER_SELECTED = "EDIT_TAG_PLAYER_FILTER_SELECTED";
	public static final String CONF_EDIT_TAG_EVENT_FILTER_SELECTED = "EDIT_TAG_EVENT_FILTER_SELECTED";
	public static final String CONF_EDIT_TAG_OPENING_FILTER_SELECTED = "EDIT_TAG_OPENING_FILTER_SELECTED";
	public static final String CONF_EDIT_TAG_TIME_AND_DATE_FILTER_SELECTED = "EDIT_TAG_TIME_AND_DATE_FILTER_SELECTED";
	public static final String CONF_EDIT_TAG_TIME_CONTROL_FILTER_SELECTED = "EDIT_TAG_TIME_CONTROL_FILTER_SELECTED";
	public static final String CONF_EDIT_TAG_GAME_CONCLUSION_FILTER_SELECTED = "EDIT_TAG_GAME_CONCLUSION_FILTER_SELECTED";
	public static final String CONF_EDIT_TAG_MISCELLANEOUS_FILTER_SELECTED = "EDIT_TAG_MISCELLANEOUS_FILTER_SELECTED";
	public static final String CONF_EDIT_TAG_ALTERNATIVE_STARTING_POSITION_FILTER_SELECTED = "EDIT_TAG_ALTERNATIVE_STARTING_POSITION_FILTER_SELECTED";
	public static final String CONF_EDIT_TAG_WINDOW_ALWAYS_ON_TOP = "EDIT_TAG_WINDOW_ALWAYS_ON_TOP";
	public static final String CONF_EDIT_TAG_FILTER_ITEM_FOR_TAG = "EDIT_TAG_FILTER_ITEM_FOR_TAG";
	
	public static final String CONF_STRING_TO_CONFIGURE_LANGUAGE_TO_SHOW_GAMES = "STRING_TO_CONFIGURE_LANGUAGE_TO_SHOW_GAMES";
	public static final String CONF_STRING_TO_CONFIGURE_LANGUAGE_TO_PARSE_TEXT_OF_GAMES_FROM = "STRING_TO_CONFIGURE_LANGUAGE_TO_PARSE_TEXT_OF_GAMES_FROM";
	
	public static final String CONF_SHOW_NAGS = "SHOW_NAGS";
	public static final String CONF_DETACHED_GAME_WINDOWS_ALWAYS_ON_TOP = "DETACHED_GAME_WINDOWS_ALWAYS_ON_TOP";

	public static final String CONF_LICENSES_HAVE_BEEN_ACCEPTED = "LICENSES_HAVE_BEEN_ACCEPTED";

	private static ApplicationConfiguration a_instance = null;

	private ApplicationConfiguration()
	{
//		super( sa_MAIN_FOLDER, sa_APPLICATION_NAME, APPLICATION_GROUP, null, GLOBAL_CONF_FILE_NAME );
		super( sa_MAIN_FOLDER, sa_APPLICATION_NAME + "/" + VERSION, sa_CONFIGURATION_GROUP, null, GLOBAL_CONF_FILE_NAME );
	}

	public static ApplicationConfiguration create( String applicationName )
	{
		sa_APPLICATION_NAME = applicationName;
		a_instance = new ApplicationConfiguration();

		return( a_instance );
	}

	public static ApplicationConfiguration instance()
	{
/*		if( a_instance == null )
		{
			a_instance = new ApplicationConfiguration();
			a_instance.loadDefaultProperties();
		}
*/		
		return( a_instance );
	}

	public Properties M_getDefaultProperties( String language )
	{
		Properties result = super.M_getDefaultProperties( language );

		result.setProperty(CONF_HAS_TO_SHOW_COMMENTS, "1" );
		result.setProperty(CONF_HAS_TO_SHOW_SEGMENTS, "0" );
		result.setProperty(CONF_LANGUAGE, ES_LANGUAGE );
		result.setProperty(CONF_ADDITIONAL_LANGUAGE, "CAT" );
		result.setProperty(CONF_EDIT_COMMENT_WINDOW_ALWAYS_ON_TOP, "1" );

		result.setProperty(CONF_EDIT_TAG_MANDATORY_FILTER_SELECTED, "1" );
		result.setProperty(CONF_EDIT_TAG_PLAYER_FILTER_SELECTED, "1" );
		result.setProperty(CONF_EDIT_TAG_EVENT_FILTER_SELECTED, "1" );
		result.setProperty(CONF_EDIT_TAG_OPENING_FILTER_SELECTED, "1" );
		result.setProperty(CONF_EDIT_TAG_TIME_AND_DATE_FILTER_SELECTED, "1" );
		result.setProperty(CONF_EDIT_TAG_TIME_CONTROL_FILTER_SELECTED, "1" );
		result.setProperty(CONF_EDIT_TAG_GAME_CONCLUSION_FILTER_SELECTED, "1" );
		result.setProperty(CONF_EDIT_TAG_ALTERNATIVE_STARTING_POSITION_FILTER_SELECTED, "1" );
		result.setProperty(CONF_EDIT_TAG_MISCELLANEOUS_FILTER_SELECTED, "1" );
		result.setProperty(CONF_EDIT_TAG_WINDOW_ALWAYS_ON_TOP, "1" );

		result.setProperty(CONF_STRING_TO_CONFIGURE_LANGUAGE_TO_SHOW_GAMES, "Spanish" );
		result.setProperty(CONF_STRING_TO_CONFIGURE_LANGUAGE_TO_PARSE_TEXT_OF_GAMES_FROM, "Spanish" );

		result.setProperty(CONF_SHOW_NAGS, "1" );
		result.setProperty(CONF_DETACHED_GAME_WINDOWS_ALWAYS_ON_TOP, "1" );
		result.setProperty(CONF_LICENSES_HAVE_BEEN_ACCEPTED, "0" );

		return( result );
	}

	public ChessLanguageConfiguration getChessLanguageConfiguration( String language )
	{
		ChessLanguageConfiguration result = EnglishChessConfiguration.instance();
		
		if( language != null )
		{
			result = ChessLanguageConfiguration.getConfiguration(language);
			if( result == null )
				result = ChessLanguageConfiguration.createConfiguration(language, language);		// in case language was not a name of a language, it would take the string with the piece codes of the language.
		}

		return( ChessLanguageConfiguration.getConfiguration(language) );
	}


	@Override
	public String getDefaultLanguageConfigurationFolder( String language )
	{
		String result = System.getProperty("user.home") + File.separator  +
						getConfigurationMainFolder() + File.separator +
						getApplicationNameFolder() + File.separator +
						getApplicationGroup() + File.separator +
						language;

		return( result );
	}

	@Override
	public String getDefaultLanguage()
	{
		return( EN_LANGUAGE );
	}

	@Override
	public String getApplicationNameFolder()
	{
		return( getApplicationName() + "/" + getApplicationVersion() );
	}

	@Override
	public String getApplicationVersion()
	{
		return( VERSION );
	}

	@Override
	public boolean getLicensesHaveBeenAccepted()
	{
		return( this.M_getBoolParamConfiguration(CONF_LICENSES_HAVE_BEEN_ACCEPTED) );
	}

	@Override
	public void setLicencesHaveBeenAccepted( boolean value )
	{
		this.M_setBoolParamConfiguration( CONF_LICENSES_HAVE_BEEN_ACCEPTED, value );
	}

	@Override
	public String getInternationalPropertiesPathInJar()
	{
		return( sa_PROPERTIES_PATH_IN_JAR );
	}
	
	public String getAdditionalLanguage()
	{
		return( M_getStrParamConfiguration( CONF_ADDITIONAL_LANGUAGE ) );
	}
	
	public void setAdditionalLanguage( String value )
	{
		M_setStrParamConfiguration( CONF_ADDITIONAL_LANGUAGE, value );
	}

	@Override
	public String getConfigurationMainFolder()
	{
		return( sa_MAIN_FOLDER );
	}

	@Override
	public String getApplicationName()
	{
		return( sa_APPLICATION_NAME );
	}

	@Override
	public String getApplicationGroup()
	{
		return( sa_CONFIGURATION_GROUP );
	}

	@Override
	public String getResourceNameForApplicationIcon()
	{
		return( RESOURCE_FOR_APPLICATION_ICON );
	}

	@Override
	public ChessLanguageConfiguration getChessLanguageConfigurationToShow()
	{
		return( getChessLanguageConfiguration( getConfigurationOfChessLanguageToShow() ) );
	}

	@Override
	public ChessLanguageConfiguration getChessLanguageConfigurationToParseTextFrom()
	{
		return( getChessLanguageConfiguration( getConfigurationOfChessLanguageToParseTextFrom( ) ) );
	}
/*
	protected ChessMoveNotation buildChessMoveNotationForView()
	{
		return( new ChessMoveAlgebraicNotation(getChessLanguageConfiguration()) );
	}
	
	protected ChessMoveMatcher buildChessMoveMatcherForView()
	{
		return( new ChessMoveAlgebraicNotation(getChessLanguageConfiguration()) );
	}
*/	
	/**
	 *
	 * @return
	 */
//	@Override
//	public ChessMoveNotation getChessMoveNotationForView()
//	{
//		if( _chessMoveNotationForView == null )
//		{
//			_chessMoveNotationForView = buildChessMoveNotationForView();
//		}
//		return( _chessMoveNotationForView );
//	}

	/**
	 *
	 * @return
	 */
//	@Override
//	public ChessMoveMatcher getChessMoveMatcherForView()
//	{
//		if( _chessMoveMatcherForView == null )
//		{
//			_chessMoveMatcherForView = buildChessMoveMatcherForView();
//		}
//		return( _chessMoveMatcherForView );
//	}

	@Override
	public boolean getHasToShowComments()
	{
		return( M_getBoolParamConfiguration(CONF_HAS_TO_SHOW_COMMENTS) );
	}

	@Override
	public void setHasToShowComments( boolean hasToShowComments )
	{
		M_setBoolParamConfiguration( CONF_HAS_TO_SHOW_COMMENTS, hasToShowComments );
	}

	@Override
	public boolean getHasToShowSegments()
	{
		return( M_getBoolParamConfiguration(CONF_HAS_TO_SHOW_SEGMENTS) );
	}

	@Override
	public boolean getEditCommentWindowAlwaysOnTop() {
		return( M_getBoolParamConfiguration(CONF_EDIT_COMMENT_WINDOW_ALWAYS_ON_TOP) );
	}

	@Override
	public void setEditCommentWindowAlwaysOnTop(boolean value)
	{
		M_setBoolParamConfiguration( CONF_EDIT_COMMENT_WINDOW_ALWAYS_ON_TOP, value );
	}

	@Override
	public boolean getMandatoryFilterSelected()
	{
		return( M_getBoolParamConfiguration( CONF_EDIT_TAG_MANDATORY_FILTER_SELECTED ) );
	}

	@Override
	public boolean getPlayerFilterSelected()
	{
		return( M_getBoolParamConfiguration( CONF_EDIT_TAG_PLAYER_FILTER_SELECTED ) );
	}

	@Override
	public boolean getEventFilterSelected()
	{
		return( M_getBoolParamConfiguration( CONF_EDIT_TAG_EVENT_FILTER_SELECTED ) );
	}

	@Override
	public boolean getOpeningFilterSelected()
	{
		return( M_getBoolParamConfiguration( CONF_EDIT_TAG_OPENING_FILTER_SELECTED ) );
	}

	@Override
	public boolean getTimeAndDateFilterSelected()
	{
		return( M_getBoolParamConfiguration( CONF_EDIT_TAG_TIME_AND_DATE_FILTER_SELECTED ) );
	}

	@Override
	public boolean getTimeControlFilterSelected()
	{
		return( M_getBoolParamConfiguration( CONF_EDIT_TAG_TIME_CONTROL_FILTER_SELECTED ) );
	}

	@Override
	public boolean getGameConclusionFilterSelected()
	{
		return( M_getBoolParamConfiguration( CONF_EDIT_TAG_GAME_CONCLUSION_FILTER_SELECTED ) );
	}

	@Override
	public boolean getMiscellaneousFilterSelected()
	{
		return( M_getBoolParamConfiguration( CONF_EDIT_TAG_MISCELLANEOUS_FILTER_SELECTED ) );
	}

	@Override
	public boolean getAlternativeStartingFilterSelected()
	{
		return( M_getBoolParamConfiguration( CONF_EDIT_TAG_ALTERNATIVE_STARTING_POSITION_FILTER_SELECTED ) );
	}

	@Override
	public String getFilterItemForTAGs(int index)
	{
		return( M_getStrParamConfiguration( CONF_EDIT_TAG_FILTER_ITEM_FOR_TAG + "_" + index ) );
	}

	@Override
	public boolean getEditTAGwindowAlwaysOnTop()
	{
		return( M_getBoolParamConfiguration( CONF_EDIT_TAG_WINDOW_ALWAYS_ON_TOP ) );
	}

	@Override
	public void setMandatoryFilterSelected(boolean value)
	{
		M_setBoolParamConfiguration( CONF_EDIT_TAG_MANDATORY_FILTER_SELECTED, value );
	}

	@Override
	public void setPlayerFilterSelected(boolean value)
	{
		M_setBoolParamConfiguration( CONF_EDIT_TAG_PLAYER_FILTER_SELECTED, value );
	}

	@Override
	public void setEventFilterSelected(boolean value)
	{
		M_setBoolParamConfiguration( CONF_EDIT_TAG_EVENT_FILTER_SELECTED, value );
	}

	@Override
	public void setOpeningFilterSelected(boolean value)
	{
		M_setBoolParamConfiguration( CONF_EDIT_TAG_OPENING_FILTER_SELECTED, value );
	}

	@Override
	public void setTimeAndDateFilterSelected(boolean value)
	{
		M_setBoolParamConfiguration( CONF_EDIT_TAG_TIME_AND_DATE_FILTER_SELECTED, value );
	}

	@Override
	public void setTimeControlFilterSelected(boolean value)
	{
		M_setBoolParamConfiguration( CONF_EDIT_TAG_TIME_CONTROL_FILTER_SELECTED, value );
	}

	@Override
	public void setGameConclusionFilterSelected(boolean value)
	{
		M_setBoolParamConfiguration( CONF_EDIT_TAG_GAME_CONCLUSION_FILTER_SELECTED, value );
	}

	@Override
	public void setMiscellaneousFilterSelected(boolean value)
	{
		M_setBoolParamConfiguration( CONF_EDIT_TAG_MISCELLANEOUS_FILTER_SELECTED, value );
	}

	@Override
	public void setAlternativeStartingFilterSelected(boolean value)
	{
		M_setBoolParamConfiguration( CONF_EDIT_TAG_ALTERNATIVE_STARTING_POSITION_FILTER_SELECTED, value );
	}


	@Override
	public void setFilterItemForTAGs(int index, String valueOfFilter)
	{
		M_setStrParamConfiguration( CONF_EDIT_TAG_FILTER_ITEM_FOR_TAG + "_" + index, valueOfFilter );
	}

	@Override
	public void setEditTagWindowAlwaysOnTop(boolean value)
	{
		M_setBoolParamConfiguration( CONF_EDIT_TAG_WINDOW_ALWAYS_ON_TOP, value );
	}

	@Override
	public String getConfigurationOfChessLanguageToShow() {
		return( M_getStrParamConfiguration( CONF_STRING_TO_CONFIGURE_LANGUAGE_TO_SHOW_GAMES ) );
	}

	@Override
	public String getConfigurationOfChessLanguageToParseTextFrom() {
		return( M_getStrParamConfiguration( CONF_STRING_TO_CONFIGURE_LANGUAGE_TO_PARSE_TEXT_OF_GAMES_FROM ) );
	}

	@Override
	public void setConfigurationOfChessLanguageToShow(String value) {
		M_setStrParamConfiguration( CONF_STRING_TO_CONFIGURE_LANGUAGE_TO_SHOW_GAMES, value );
	}

	@Override
	public void setConfigurationOfChessLanguageToParseTextFrom(String value) {
		M_setStrParamConfiguration( CONF_STRING_TO_CONFIGURE_LANGUAGE_TO_PARSE_TEXT_OF_GAMES_FROM, value );
	}

	@Override
	public boolean getHasToShowNAGs()
	{
		return( M_getBoolParamConfiguration( CONF_SHOW_NAGS ) );
	}

	@Override
	public void setHasToShowNAGs(boolean value)
	{
		this.M_setBoolParamConfiguration( CONF_SHOW_NAGS, value );
	}

	@Override
	public boolean getDetachedGameWindowsAlwaysOnTop() {
		return( M_getBoolParamConfiguration( CONF_DETACHED_GAME_WINDOWS_ALWAYS_ON_TOP ) );
	}

	@Override
	public void setDetachedGameWindowsAlwaysOnTop(boolean value) {
		this.M_setBoolParamConfiguration( CONF_DETACHED_GAME_WINDOWS_ALWAYS_ON_TOP, value );
	}
/*
	@Override
	protected void changeLanguage_internal_simple( String language ) throws ConfigurationException
	{
		
	}
*/
}