/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.chesspdfbrowser.view.chess.initialposition;

import com.frojasg1.chesspdfbrowser.model.chess.board.ChessGamePosition;
import com.frojasg1.chesspdfbrowser.model.chess.pieces.Bishop;
import com.frojasg1.chesspdfbrowser.model.chess.pieces.ChessPiece;
import com.frojasg1.chesspdfbrowser.model.chess.pieces.Knight;
import com.frojasg1.chesspdfbrowser.model.chess.pieces.Pawn;
import com.frojasg1.chesspdfbrowser.model.chess.pieces.Queen;
import com.frojasg1.chesspdfbrowser.model.chess.pieces.Rook;
import com.frojasg1.general.desktop.view.panels.AcceptCancelRevertPanel;
import com.frojasg1.chesspdfbrowser.view.chess.images.ChessBoardImages;
import com.frojasg1.desktop.liblens.graphics.ScreenImage;
import com.frojasg1.general.desktop.ImageFunctions;
import com.frojasg1.general.desktop.mouse.AvailableCursors;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JTextField;

/**
 *
 * @author Usuario
 */
public class PiecesToChoosePanel extends javax.swing.JPanel  implements MouseListener,
														MouseMotionListener
{
	protected AcceptCancelRevertPanel _acceptPanel = null;

	protected InitialPositionDialog _parent = null;
	protected GlobalCoordinatesInDrag _globalCoordinatesInDrag = null;
	
	protected Map< Component, PieceData > _mapToPieceData = null;

	/**
	 * Creates new form PiecesToChoosePanel
	 */
	public PiecesToChoosePanel( InitialPositionDialog parent,
								GlobalCoordinatesInDrag gcid )
	{
		_parent = parent;
		_globalCoordinatesInDrag = gcid;
		_globalCoordinatesInDrag.setPiecesToChoosePanel(this);
		initComponents();

		initOwnComponents();
		initPieceButtons();

		recursiveAddListeners( this );
	}

	public AcceptCancelRevertPanel getAcceptCancelRevertPanel()
	{
		return( _acceptPanel );
	}
	
	protected void initOwnComponents()
	{
//		System.out.println( "PiecesToChoosePanel" );
		_acceptPanel = new AcceptCancelRevertPanel( _parent );

		jPanel5.add( _acceptPanel );
		_acceptPanel.setBounds( 0, 0, jPanel5.getWidth(), jPanel5.getHeight() );
	}

	public int getNumberOfPliesWithoutProgress()
	{
		int result = -1;
		try
		{
			int tmp = Integer.parseInt( jTF_movesWithoutProgress.getText() );
			if( ( tmp > -1 ) && ( tmp < 100 ) )
				result = tmp;
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}
		return( result );
	}

	public JTextField getNumberOfPliesWithoutProgressJTextField()
	{
		return( jTF_movesWithoutProgress );
	}
	
	public JTextField getEnPassantJTextField()
	{
		return( jTF_enPassantMove );
	}

	public JTextField getMoveNumberJTextField()
	{
		return( jTF_moveNumber );
	}

	public String getEnPassantMove()
	{
		return( jTF_enPassantMove.getText() );
	}

	public void setMoveNumber( int moveNumber )
	{
		if( moveNumber == 0 )	// the moveNumber only can be configured in case in the ChessGame there is not any move.
		{
			moveNumber = 1;
			jTF_moveNumber.setText( String.valueOf(moveNumber) );
			jTF_moveNumber.setEditable( true );
		}
		else
		{
			jTF_moveNumber.setText( String.valueOf(moveNumber) );
			jTF_moveNumber.setEditable( false );
		}
	}

	public boolean isWhiteToPlay()
	{
		return( jRB_whiteToPlay.isSelected() );
	}

	public boolean whiteCanCastleKingside()
	{
		return( jcb_whi_castle_king_side.isSelected() );
	}

	public boolean whiteCanCastleQueenside()
	{
		return( jcb_whi_castle_queen_side.isSelected() );
	}

	public boolean blackCanCastleKingside()
	{
		return( jcb_bla_castle_king_side.isSelected() );
	}

	public boolean blackCanCastleQueenside()
	{
		return( jcb_bla_castle_queen_side.isSelected() );
	}

	public void setWhiteCanCastleKingsSide( boolean value )
	{
		setCastleCheckboxSelected( jcb_whi_castle_king_side, value );
	}
	
	public void setWhiteCanCastleQueenSide( boolean value )
	{
		setCastleCheckboxSelected( jcb_whi_castle_queen_side, value );
	}
	
	public void setBlackCanCastleKingsSide( boolean value )
	{
		setCastleCheckboxSelected( jcb_bla_castle_king_side, value );
	}
	
	public void setBlackCanCastleQueenSide( Boolean value )
	{
		setCastleCheckboxSelected( jcb_bla_castle_queen_side, value );
	}
	
	public void setCastleCheckboxSelected( JCheckBox jcb, Boolean value )
	{
		if( value == null )
			value = true;

		jcb.setSelected(value);
	}
	
	public void setPliesWithoutProgress( Integer numberOfPliesWithoutProgress )
	{
		if( numberOfPliesWithoutProgress == null )
			numberOfPliesWithoutProgress = 0;

		jTF_movesWithoutProgress.setText( String.valueOf(numberOfPliesWithoutProgress) );
	}
	
	public void setWhiteToPlay()
	{
		jRB_whiteToPlay.setSelected(true);
	}

	public void setBlackToPlay()
	{
		jRB_blackToPlay.setSelected(true);
	}

	public void setEnPassantMove( String enPassantMove )
	{
		if( enPassantMove == null )
			enPassantMove = "";

		jTF_enPassantMove.setText( enPassantMove );
	}

	protected void addImageToButton( JButton button, String resourceName )
	{
		try
		{
			BufferedImage img = ImageIO.read(ClassLoader.getSystemClassLoader().getResourceAsStream(resourceName));
			Image resizedImage = ImageFunctions.resizeImage( img, button.getWidth()-2, button.getHeight()-2, 0xFFB0B0B0, 0xFFF8E2E2, null );
			button.setIcon(new ImageIcon(resizedImage));
			button.setMargin(new Insets(1, 1, 1, 1));
		}
		catch (IOException ex)
		{
			ex.printStackTrace();
		}
	}

	protected void initPieceButtons( )
	{
		addImageToButton(jB_white_Queen, "com/frojasg1/chesspdfbrowser/resources/chess/images/processed.white.queen.png");
		addImageToButton(jB_white_Rook, "com/frojasg1/chesspdfbrowser/resources/chess/images/processed.white.rook.png");
		addImageToButton(jB_white_Bishop, "com/frojasg1/chesspdfbrowser/resources/chess/images/processed.white.bishop.png");
		addImageToButton(jB_white_Knight, "com/frojasg1/chesspdfbrowser/resources/chess/images/processed.white.knight.png");
		addImageToButton(jB_white_Pawn, "com/frojasg1/chesspdfbrowser/resources/chess/images/processed.white.pawn.png");
		addImageToButton(jB_black_Queen, "com/frojasg1/chesspdfbrowser/resources/chess/images/processed.black.queen.png");
		addImageToButton(jB_black_Rook, "com/frojasg1/chesspdfbrowser/resources/chess/images/processed.black.rook.png");
		addImageToButton(jB_black_Bishop, "com/frojasg1/chesspdfbrowser/resources/chess/images/processed.black.bishop.png");
		addImageToButton(jB_black_Knight, "com/frojasg1/chesspdfbrowser/resources/chess/images/processed.black.knight.png");
		addImageToButton(jB_black_Pawn, "com/frojasg1/chesspdfbrowser/resources/chess/images/processed.black.pawn.png");

		_mapToPieceData = new HashMap<Component, PieceData>();
		_mapToPieceData.put( jB_white_Queen, new PieceData( ChessPiece.WHITE, Queen.PIECE_TYPE_CODE ) );
		_mapToPieceData.put( jB_white_Rook, new PieceData( ChessPiece.WHITE, Rook.PIECE_TYPE_CODE ) );
		_mapToPieceData.put( jB_white_Bishop, new PieceData( ChessPiece.WHITE, Bishop.PIECE_TYPE_CODE ) );
		_mapToPieceData.put( jB_white_Knight, new PieceData( ChessPiece.WHITE, Knight.PIECE_TYPE_CODE ) );
		_mapToPieceData.put( jB_white_Pawn, new PieceData( ChessPiece.WHITE, Pawn.PIECE_TYPE_CODE ) );
		_mapToPieceData.put( jB_black_Queen, new PieceData( ChessPiece.BLACK, Queen.PIECE_TYPE_CODE ) );
		_mapToPieceData.put( jB_black_Rook, new PieceData( ChessPiece.BLACK, Rook.PIECE_TYPE_CODE ) );
		_mapToPieceData.put( jB_black_Bishop, new PieceData( ChessPiece.BLACK, Bishop.PIECE_TYPE_CODE ) );
		_mapToPieceData.put( jB_black_Knight, new PieceData( ChessPiece.BLACK, Knight.PIECE_TYPE_CODE ) );
		_mapToPieceData.put( jB_black_Pawn, new PieceData( ChessPiece.BLACK, Pawn.PIECE_TYPE_CODE ) );
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        buttonGroup1 = new javax.swing.ButtonGroup();
        jP_contentPanel = new javax.swing.JPanel();
        jB_black_Queen = new javax.swing.JButton();
        jB_black_Rook = new javax.swing.JButton();
        jB_black_Bishop = new javax.swing.JButton();
        jB_black_Pawn = new javax.swing.JButton();
        jB_black_Knight = new javax.swing.JButton();
        jB_white_Pawn = new javax.swing.JButton();
        jB_white_Knight = new javax.swing.JButton();
        jB_white_Bishop = new javax.swing.JButton();
        jB_white_Rook = new javax.swing.JButton();
        jB_white_Queen = new javax.swing.JButton();
        jCB_whitePlaysFromBottom = new javax.swing.JCheckBox();
        jPanel1 = new javax.swing.JPanel();
        jRB_whiteToPlay = new javax.swing.JRadioButton();
        jRB_blackToPlay = new javax.swing.JRadioButton();
        jcb_whi_castle_king_side = new javax.swing.JCheckBox();
        jcb_whi_castle_queen_side = new javax.swing.JCheckBox();
        jcb_bla_castle_king_side = new javax.swing.JCheckBox();
        jcb_bla_castle_queen_side = new javax.swing.JCheckBox();
        jTF_movesWithoutProgress = new javax.swing.JTextField();
        jL_pliesWIthoutProgress = new javax.swing.JLabel();
        jB_initialPosition = new javax.swing.JButton();
        jB_onlyKings = new javax.swing.JButton();
        jSeparator1 = new javax.swing.JSeparator();
        jTF_enPassantMove = new javax.swing.JTextField();
        jL_enPassantSquare = new javax.swing.JLabel();
        jPanel5 = new javax.swing.JPanel();
        jB_pasteFENposition = new javax.swing.JButton();
        jTF_FENstring = new javax.swing.JTextField();
        jL_FENstring = new javax.swing.JLabel();
        jTF_moveNumber = new javax.swing.JTextField();
        jL_moveNumber = new javax.swing.JLabel();

        setToolTipText("");
        setMinimumSize(new java.awt.Dimension(658, 240));
        setPreferredSize(new java.awt.Dimension(658, 240));
        setLayout(null);

        jP_contentPanel.setMinimumSize(new java.awt.Dimension(658, 240));
        jP_contentPanel.setPreferredSize(new java.awt.Dimension(658, 240));
        jP_contentPanel.setLayout(null);

        jB_black_Queen.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        jB_black_Queen.setFocusable(false);
        jP_contentPanel.add(jB_black_Queen);
        jB_black_Queen.setBounds(10, 160, 40, 40);

        jB_black_Rook.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        jB_black_Rook.setFocusable(false);
        jP_contentPanel.add(jB_black_Rook);
        jB_black_Rook.setBounds(50, 160, 40, 40);

        jB_black_Bishop.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        jB_black_Bishop.setFocusable(false);
        jP_contentPanel.add(jB_black_Bishop);
        jB_black_Bishop.setBounds(90, 160, 40, 40);

        jB_black_Pawn.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        jB_black_Pawn.setFocusable(false);
        jP_contentPanel.add(jB_black_Pawn);
        jB_black_Pawn.setBounds(170, 160, 40, 40);

        jB_black_Knight.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        jB_black_Knight.setFocusable(false);
        jP_contentPanel.add(jB_black_Knight);
        jB_black_Knight.setBounds(130, 160, 40, 40);

        jB_white_Pawn.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        jB_white_Pawn.setFocusable(false);
        jP_contentPanel.add(jB_white_Pawn);
        jB_white_Pawn.setBounds(170, 110, 40, 40);

        jB_white_Knight.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        jB_white_Knight.setFocusable(false);
        jP_contentPanel.add(jB_white_Knight);
        jB_white_Knight.setBounds(130, 110, 40, 40);

        jB_white_Bishop.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        jB_white_Bishop.setFocusable(false);
        jP_contentPanel.add(jB_white_Bishop);
        jB_white_Bishop.setBounds(90, 110, 40, 40);

        jB_white_Rook.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        jB_white_Rook.setFocusable(false);
        jP_contentPanel.add(jB_white_Rook);
        jB_white_Rook.setBounds(50, 110, 40, 40);

        jB_white_Queen.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        jB_white_Queen.setFocusable(false);
        jP_contentPanel.add(jB_white_Queen);
        jB_white_Queen.setBounds(10, 110, 40, 40);

        jCB_whitePlaysFromBottom.setSelected(true);
        jCB_whitePlaysFromBottom.setText("white plays from bottom");
        jCB_whitePlaysFromBottom.setName("jCB_whitePlaysFromBottom"); // NOI18N
        jCB_whitePlaysFromBottom.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jcb_whi_castle_king_sideActionPerformed(evt);
            }
        });
        jP_contentPanel.add(jCB_whitePlaysFromBottom);
        jCB_whitePlaysFromBottom.setBounds(230, 10, 210, 23);

        jPanel1.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        jPanel1.setLayout(null);

        buttonGroup1.add(jRB_whiteToPlay);
        jRB_whiteToPlay.setSelected(true);
        jRB_whiteToPlay.setText("white to play");
        jRB_whiteToPlay.setName("jRB_whiteToPlay"); // NOI18N
        jRB_whiteToPlay.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jcb_whi_castle_king_sideActionPerformed(evt);
            }
        });
        jPanel1.add(jRB_whiteToPlay);
        jRB_whiteToPlay.setBounds(10, 10, 150, 23);

        buttonGroup1.add(jRB_blackToPlay);
        jRB_blackToPlay.setText("black to play");
        jRB_blackToPlay.setName("jRB_blackToPlay"); // NOI18N
        jRB_blackToPlay.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jcb_whi_castle_king_sideActionPerformed(evt);
            }
        });
        jPanel1.add(jRB_blackToPlay);
        jRB_blackToPlay.setBounds(10, 30, 150, 23);

        jP_contentPanel.add(jPanel1);
        jPanel1.setBounds(440, 0, 170, 60);

        jcb_whi_castle_king_side.setSelected(true);
        jcb_whi_castle_king_side.setText("white can castle king side");
        jcb_whi_castle_king_side.setName("jcb_whi_castle_king_side"); // NOI18N
        jcb_whi_castle_king_side.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jcb_whi_castle_king_sideActionPerformed(evt);
            }
        });
        jP_contentPanel.add(jcb_whi_castle_king_side);
        jcb_whi_castle_king_side.setBounds(440, 70, 220, 23);

        jcb_whi_castle_queen_side.setSelected(true);
        jcb_whi_castle_queen_side.setText("white can castle queen side");
        jcb_whi_castle_queen_side.setName("jcb_whi_castle_queen_side"); // NOI18N
        jcb_whi_castle_queen_side.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jcb_whi_castle_king_sideActionPerformed(evt);
            }
        });
        jP_contentPanel.add(jcb_whi_castle_queen_side);
        jcb_whi_castle_queen_side.setBounds(440, 100, 220, 23);

        jcb_bla_castle_king_side.setSelected(true);
        jcb_bla_castle_king_side.setText("black can castle king side");
        jcb_bla_castle_king_side.setName("jcb_bla_castle_king_side"); // NOI18N
        jcb_bla_castle_king_side.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jcb_whi_castle_king_sideActionPerformed(evt);
            }
        });
        jP_contentPanel.add(jcb_bla_castle_king_side);
        jcb_bla_castle_king_side.setBounds(440, 130, 220, 23);

        jcb_bla_castle_queen_side.setSelected(true);
        jcb_bla_castle_queen_side.setText("black can castle queen side");
        jcb_bla_castle_queen_side.setName("jcb_bla_castle_queen_side"); // NOI18N
        jcb_bla_castle_queen_side.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jcb_whi_castle_king_sideActionPerformed(evt);
            }
        });
        jP_contentPanel.add(jcb_bla_castle_queen_side);
        jcb_bla_castle_queen_side.setBounds(440, 160, 220, 23);

        jTF_movesWithoutProgress.setText("0");
        jTF_movesWithoutProgress.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                jTF_enPassantMoveKeyReleased(evt);
            }
        });
        jP_contentPanel.add(jTF_movesWithoutProgress);
        jTF_movesWithoutProgress.setBounds(230, 80, 20, 20);

        jL_pliesWIthoutProgress.setText("plies without progress");
        jL_pliesWIthoutProgress.setName("jL_pliesWIthoutProgress"); // NOI18N
        jL_pliesWIthoutProgress.setRequestFocusEnabled(false);
        jP_contentPanel.add(jL_pliesWIthoutProgress);
        jL_pliesWIthoutProgress.setBounds(260, 80, 170, 20);

        jB_initialPosition.setText("Set initial position");
        jB_initialPosition.setName("jB_initialPosition"); // NOI18N
        jB_initialPosition.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jB_initialPositionActionPerformed(evt);
            }
        });
        jP_contentPanel.add(jB_initialPosition);
        jB_initialPosition.setBounds(10, 10, 200, 23);

        jB_onlyKings.setText("Leave only kings");
        jB_onlyKings.setName("jB_onlyKings"); // NOI18N
        jB_onlyKings.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jB_onlyKingsActionPerformed(evt);
            }
        });
        jP_contentPanel.add(jB_onlyKings);
        jB_onlyKings.setBounds(10, 40, 200, 23);

        jSeparator1.setOrientation(javax.swing.SwingConstants.VERTICAL);
        jP_contentPanel.add(jSeparator1);
        jSeparator1.setBounds(220, 10, 10, 190);

        jTF_enPassantMove.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                jTF_enPassantMoveKeyReleased(evt);
            }
        });
        jP_contentPanel.add(jTF_enPassantMove);
        jTF_enPassantMove.setBounds(230, 110, 20, 20);

        jL_enPassantSquare.setText("En passant square");
        jL_enPassantSquare.setName("jL_enPassantSquare"); // NOI18N
        jP_contentPanel.add(jL_enPassantSquare);
        jL_enPassantSquare.setBounds(260, 110, 180, 20);

        jPanel5.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 102, 204)));
        jPanel5.setPreferredSize(new java.awt.Dimension(130, 50));
        jPanel5.setLayout(null);
        jP_contentPanel.add(jPanel5);
        jPanel5.setBounds(270, 150, 130, 50);

        jB_pasteFENposition.setText("Paste FEN position");
        jB_pasteFENposition.setName("jB_pasteFENposition"); // NOI18N
        jB_pasteFENposition.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jB_pasteFENpositionActionPerformed(evt);
            }
        });
        jP_contentPanel.add(jB_pasteFENposition);
        jB_pasteFENposition.setBounds(10, 70, 200, 23);

        jTF_FENstring.setEditable(false);
        jTF_FENstring.setFont(new java.awt.Font("Tahoma", 0, 10)); // NOI18N
        jTF_FENstring.setForeground(new java.awt.Color(51, 102, 255));
        jP_contentPanel.add(jTF_FENstring);
        jTF_FENstring.setBounds(130, 210, 520, 19);

        jL_FENstring.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jL_FENstring.setText("FEN string :");
        jL_FENstring.setName("jL_FENstring"); // NOI18N
        jP_contentPanel.add(jL_FENstring);
        jL_FENstring.setBounds(0, 210, 120, 14);
        jP_contentPanel.add(jTF_moveNumber);
        jTF_moveNumber.setBounds(230, 50, 30, 20);

        jL_moveNumber.setText("Move number");
        jL_moveNumber.setName("jL_moveNumber"); // NOI18N
        jP_contentPanel.add(jL_moveNumber);
        jL_moveNumber.setBounds(270, 50, 160, 20);

        add(jP_contentPanel);
        jP_contentPanel.setBounds(0, 0, 658, 240);
    }// </editor-fold>//GEN-END:initComponents

    private void jB_initialPositionActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_initialPositionActionPerformed
        // TODO add your handling code here:

		_globalCoordinatesInDrag.setPosition( ChessGamePosition.getInitialPosition() );

    }//GEN-LAST:event_jB_initialPositionActionPerformed

    private void jB_onlyKingsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_onlyKingsActionPerformed
        // TODO add your handling code here:

		_globalCoordinatesInDrag.setPosition( ChessGamePosition.getOnlyKingsPosition() );

    }//GEN-LAST:event_jB_onlyKingsActionPerformed

    private void jB_pasteFENpositionActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_pasteFENpositionActionPerformed
        // TODO add your handling code here:

		_globalCoordinatesInDrag.pasteFenPosition();

    }//GEN-LAST:event_jB_pasteFENpositionActionPerformed

    private void jcb_whi_castle_king_sideActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jcb_whi_castle_king_sideActionPerformed
        // TODO add your handling code here:

		_globalCoordinatesInDrag.newChangeInPosition();

    }//GEN-LAST:event_jcb_whi_castle_king_sideActionPerformed

    private void jTF_enPassantMoveKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jTF_enPassantMoveKeyReleased
        // TODO add your handling code here:

		_globalCoordinatesInDrag.newChangeInPosition();
		
    }//GEN-LAST:event_jTF_enPassantMoveKeyReleased


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup buttonGroup1;
    private javax.swing.JButton jB_black_Bishop;
    private javax.swing.JButton jB_black_Knight;
    private javax.swing.JButton jB_black_Pawn;
    private javax.swing.JButton jB_black_Queen;
    private javax.swing.JButton jB_black_Rook;
    private javax.swing.JButton jB_initialPosition;
    private javax.swing.JButton jB_onlyKings;
    private javax.swing.JButton jB_pasteFENposition;
    private javax.swing.JButton jB_white_Bishop;
    private javax.swing.JButton jB_white_Knight;
    private javax.swing.JButton jB_white_Pawn;
    private javax.swing.JButton jB_white_Queen;
    private javax.swing.JButton jB_white_Rook;
    private javax.swing.JCheckBox jCB_whitePlaysFromBottom;
    private javax.swing.JLabel jL_FENstring;
    private javax.swing.JLabel jL_enPassantSquare;
    private javax.swing.JLabel jL_moveNumber;
    private javax.swing.JLabel jL_pliesWIthoutProgress;
    private javax.swing.JPanel jP_contentPanel;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JRadioButton jRB_blackToPlay;
    private javax.swing.JRadioButton jRB_whiteToPlay;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JTextField jTF_FENstring;
    private javax.swing.JTextField jTF_enPassantMove;
    private javax.swing.JTextField jTF_moveNumber;
    private javax.swing.JTextField jTF_movesWithoutProgress;
    private javax.swing.JCheckBox jcb_bla_castle_king_side;
    private javax.swing.JCheckBox jcb_bla_castle_queen_side;
    private javax.swing.JCheckBox jcb_whi_castle_king_side;
    private javax.swing.JCheckBox jcb_whi_castle_queen_side;
    // End of variables declaration//GEN-END:variables


	protected void recursiveSetCursor( Cursor cursor )
	{
		if( ! getCursor().getName().equals( cursor.getName() ) )
		{
			recursiveSetCursor( this, cursor );
		}
	}
	
	protected void recursiveSetCursor( Component comp, Cursor cursor )
	{
		comp.setCursor( cursor );
		if( comp instanceof Container )
		{
			Container cont = (Container) comp;
			
			for( int ii=0; ii<cont.getComponentCount(); ii++ )
				recursiveSetCursor( cont.getComponent(ii), cursor );
		}
	}

	protected void recursiveAddListeners( Component comp )
	{
		comp.addMouseMotionListener( this );
		comp.addMouseListener( this );
		if( comp instanceof Container )
		{
			Container cont = (Container) comp;
			
			for( int ii=0; ii<cont.getComponentCount(); ii++ )
				recursiveAddListeners( cont.getComponent(ii) );
		}
	}
	
	@Override
	public void mouseClicked(MouseEvent me)
	{
	}

	@Override
	public void mousePressed(MouseEvent me)
	{
		if( me.getButton() == MouseEvent.BUTTON1 ) 
		{
			Component component = me.getComponent();

			PieceData pd = _mapToPieceData.get( component );
			if( pd != null )
			{
				ChessPiece cp = null;
				try
				{
					cp = ChessPiece.createPiece( _globalCoordinatesInDrag.getChessBoard(), pd.getPieceType(), pd.getColor() );
				}
				catch( Throwable th )
				{
					th.printStackTrace();
				}

				if( cp != null )
				{
					_globalCoordinatesInDrag.setDraggedPiece(cp);
					_globalCoordinatesInDrag.setIsInDrag(true);
					_globalCoordinatesInDrag.setNewDragPosition( me.getLocationOnScreen() );
					recursiveSetCursor( AvailableCursors._handCursor );

					repaint();
	//				_globalCoordinatesInDrag.repaint( false );
				}
			}
		}
	}

	@Override
	public void mouseReleased(MouseEvent me)
	{
		if( me.getButton() == MouseEvent.BUTTON1 ) 
		{
			_globalCoordinatesInDrag.setNewDragPosition( me.getLocationOnScreen() );

			recursiveSetCursor( AvailableCursors._defaultCursor );
			_globalCoordinatesInDrag.mouseReleased();
		}
	}

	@Override
	public void mouseEntered(MouseEvent me)
	{
		
	}

	@Override
	public void mouseExited(MouseEvent me)
	{
	}

	@Override
	public void mouseMoved( MouseEvent evt )
	{
		if( !_globalCoordinatesInDrag.isInDrag() )
		{
			Component component = evt.getComponent();

			PieceData pd = _mapToPieceData.get( component );
			if( pd != null )
			{
				recursiveSetCursor( AvailableCursors._handCursor );
			}
			else
			{
				recursiveSetCursor( AvailableCursors._defaultCursor );
			}
		}
		else
		{
			recursiveSetCursor( AvailableCursors._defaultCursor );
		}
	}

	@Override
	public void mouseDragged( MouseEvent evt )
	{
		if( _globalCoordinatesInDrag.isInDrag() )
		{
			_globalCoordinatesInDrag.setNewDragPosition( evt.getLocationOnScreen() );
			_globalCoordinatesInDrag.repaint( false );
		}
	}

	protected void paintPieceInDrag( Graphics gc )
	{
		if( _globalCoordinatesInDrag.isInDrag() )
		{
			try
			{
				ChessPiece cp = _globalCoordinatesInDrag.getDraggedPiece();

				if( cp != null )
				{
					int currentPieceWidth = _globalCoordinatesInDrag.getPieceWidth();
					Point dragPoint = _globalCoordinatesInDrag.getDragPosition( this );
					Point upperLeftCorner = new Point(	(int) ( dragPoint.getX() - currentPieceWidth/2 ),
														(int) ( dragPoint.getY() - currentPieceWidth/2 ) );
					BufferedImage bi = ChessBoardImages.instance().getSemiTransparentPieceImage( cp.getPieceCode(), currentPieceWidth );
					ImageFunctions.paintClippedImage(this, gc, bi, upperLeftCorner);
				}
			}
			catch( Throwable th )
			{
				th.printStackTrace();
			}
		}
	}

	@Override
	public void paintComponent(Graphics gc)
	{
		paintPanel( gc );
	}

	@Override
	public void paint(Graphics gc)
	{
		paintPanel( gc );
	}

	public void paintPanel( Graphics gc )
	{
		BufferedImage bi = ScreenImage.createImage( jP_contentPanel );
		Graphics gc1 = bi.getGraphics();

		paintPieceInDrag( gc1 );

		gc.drawImage( bi, 0, 0, bi.getWidth(), bi.getHeight(), 0, 0, bi.getWidth(), bi.getHeight(), null );
	}
	
	@Override
	public void setSize( Dimension dim )
	{
		super.setSize( dim );

		jP_contentPanel.setSize( dim );
	}

	@Override
	public void setSize( int width, int height )
	{
		super.setSize( width, height );

		jP_contentPanel.setSize( width, height );
	}
	
	@Override
	public void setBounds( Rectangle rect )
	{
		super.setBounds( rect );

		jP_contentPanel.setSize( rect.getSize() );
	}
	
	@Override
	public void setBounds( int xx, int yy, int width, int height )
	{
		super.setBounds( xx, yy, width, height );

		jP_contentPanel.setSize( width, height );
	}

	public void setFenString( String fenString )
	{
		jTF_FENstring.setText( fenString );
	}

	public static class PieceData
	{
		protected int _color = -1;
		protected String _pieceType = null;
		
		public PieceData( int color, String pieceType )
		{
			_color = color;
			_pieceType = pieceType;
		}

		public int getColor()
		{
			return( _color );
		}

		public String getPieceType()
		{
			return( _pieceType );
		}
	}

}
